#ifndef DEF_H_
#define DEF_H_

// UNCOMMENT ONLY ONE! OF THESE TO COMPILE THE CODE FOR THE APPROPRIATE SENSOR

#define SENSOR_MS5534
//#define SENSOR_SCP1000
//#define SENSOR_FOSP01

// UNCOMMENT ONLY ONE! OF THESE TO SELECT PRESSURE OR ALTITUDE REGRESSION FOR VARIO
//#define USE_P_REGRESSION
#define USE_Z_REGRESSION

#define USE_SPLASH_SCREEN


////////////////////////////////////////////////////////////////////////////////////

#include <hel2.h>
#include <mygba.h>

typedef float	f32;
typedef double f64;

typedef struct FLIGHT_DATA_
   {
   int nLaunchAltM;
   int nMaxAltM;
   int nLZAltM;
   int nMaxCRCm;
   int nMaxSRCm;
   } FLIGHT_DATA;

#define POS_ALT_X             17
#define POS_ALT_Y              5
#define POS_ALTAL_X           70
#define POS_ALTAL_Y           48
#define POS_TEMP_X            70
#define POS_TEMP_Y            75
#define POS_FTIME_X           42
#define POS_FTIME_Y          130
#define POS_RTIME_X           70
#define POS_RTIME_Y          101
#define POS_FILTER_X          20
#define POS_FILTER_Y          80
#define POS_VRES_X            40
#define POS_VRES_Y            80
#define POS_LARROW_X           165
#define POS_LARROW_Y           71
#define POS_RARROW_X           219
#define POS_RARROW_Y           71

#define POS_DATE_X             5
#define POS_DATE_Y            18

#define SLIDER_SCROLL_OFFSET 177

#define LOG_X                  3
#define LOG_Y                  7

#define SEL_Y_ORIG          4

#define SEL_ALTITUDE        0
#define SEL_WINDOW_SECS     1
#define SEL_VARIO_AUDIO     2
#define SEL_SINK_THRESH     3
#define SEL_RTC_YAR         4
#define SEL_RTC_MON         5
#define SEL_RTC_DOM         6
#define SEL_RTC_DOW         7
#define SEL_RTC_HRS         8
#define SEL_RTC_MIN         9

#define VARIO_MODE_CHIRP      11
#define VARIO_MODE_VOICE      22

#define VARIO_MAX_CLIMB_RATE_LOG_CM       5000
#define VARIO_MAX_CLIMB_RATE_DPY_CM       1600
#define VARIO_CROSSOVER_CLIMB_RATE_CM      400
#define VARIO_SINKTONE_HI_CM               400
#define VARIO_CORE_CLIMB_RATE_CM            10


#define VARIO_RESOLUTION_H                  5
#define VARIO_RESOLUTION_M                  20
#define VARIO_RESOLUTION_L                  50
#define VARIO_DFL_RESOLUTION                VARIO_RESOLUTION_M
#define VARIO_MAX_RESOLUTION                50
#define VARIO_MIN_RESOLUTION                 5

#define VARIO_MIN_SINKTONE_THRESHOLD_CM      50
#define VARIO_DFL_SINKTONE_THRESHOLD_CM     250
#define VARIO_MAX_SINKTONE_THRESHOLD_CM     400

#define VARIO_TMR_TICKS                      50 // 100 mS intervals @2mS/tick

#define NVD_MAX_FLIGHTS                 100
#define NVD_DATA_BYTES                  (8+18*NVD_MAX_FLIGHTS)

#define NVD_ATO_THRESHOLD                5

#define SIM_START_ALTITUDE_M        500
#define SIM_START_TEMP_CX10         250

#define SENSOR_MAX_ALTITUDE_M         9999
#define SENSOR_MIN_ALTITUDE_M         -999

#define SENSOR_MAX_TEMPX10             999
#define SENSOR_MIN_TEMPX10            -999

#define SENSOR_DFL_WINDOW_SECS           3
#define SENSOR_MIN_WINDOW_SECS           2
#define SENSOR_MAX_WINDOW_SECS           8


#define SENSOR_KLPFC_H                  3
#define SENSOR_KLPFC_M                  8
#define SENSOR_KLPFC_L                 16
#define SENSOR_DFL_KLPFC                SENSOR_KLPFC_M
#define SENSOR_MAX_KLPFC                16
#define SENSOR_MIN_KLPFC                 1

#define SENSOR_DFL_PZTBL_SEL            50
#define SENSOR_MIN_PZTBL_SEL             0
#define SENSOR_MAX_PZTBL_SEL            99
#define SENSOR_DFL_TEMP_CORR_INX        25
#define SENSOR_T0_MIN                  -10
#define SENSOR_T0_MAX                   40

#define MODE_LOG                        11
#define MODE_SETTINGS                   22
#define MODE_RUNNING                    33
#define MODE_SIMULATION                 44

#define I2C_PORT1                        1
#define I2C_PORT2                        2

#define I2C_ID_DS1338            ((u8)0xD0)
#define I2C_ID_SCP1000           ((u8)0x22)
#define I2C_ID_FOSP01_COEFF      ((u8)0xA0)
#define I2C_ID_FOSP01_ADC        ((u8)0xEE)

#define TMR_TICK_MS                      2
#define TMR_TICKS_PER_SEC              500  // modify this if TMR_TICK_MS is changed !!

//#define TMR_REG_VAL(fsec,scaler)   ((u16)((65536.0f - (16777216/scaler)*fsec) + 0.5f))
#define TMR_REG_VAL(msec,scaler)      (65536 - ((16777216/scaler)*msec)/1000)
#define WAIT_TICK(delayMs)           (gTmr.ticks + (delayMs/TMR_TICK_MS))

#define PORT_REG                (*((vu16*)REG_RCNT))  // Link port register

#define PORT_SC                 0x0001
#define PORT_SD                 0x0002
#define PORT_SI                 0x0004
#define PORT_SO                 0x0008
#define PORT_SC_WR              0x0010
#define PORT_SD_WR              0x0020
#define PORT_SI_WR              0x0040
#define PORT_SO_WR              0x0080
#define PORT_GEN_PURPOSE        0x8000

#define CLAMP(x,min,max)    {if ((x) <= (min)) (x) = (min); else if ((x) >= (max)) (x) = (max);}
#define CORE(x,t)           {if (ABS(x) <= (t)) (x) = 0;}

#endif // DEF_H_

/*
 * $Header: $
 */

