#include "i2c.h"




int CI2C::Init(u16 pinSCL, u16 pinSDA)
    {
    m_error = 0;
    if ((PORT_REG & PORT_GEN_PURPOSE) == 0)
       PORT_REG = PORT_GEN_PURPOSE;

    m_SCLH = pinSCL;
    m_SCLW = pinSCL<<4;
    m_SDAH = pinSDA;
    m_SDAW = pinSDA<<4;
    
    PORT_REG |= m_SDAW;
    PORT_REG |= m_SDAH;        // SDA high
    PORT_REG |= m_SCLW;
    PORT_REG |= m_SCLH;        // SCL high

    I2C_DELAY();
    return 0;
    }

int CI2C::Start()
    {
    PORT_REG |= m_SCLH; //  SCL is high
    I2C_DELAY();
    PORT_REG |= m_SDAW;
    PORT_REG &= ~m_SDAH; // Pull SDA down while SCL is high ... start condition
    I2C_DELAY();
    PORT_REG &= ~m_SCLH; //  Pull SCL low for next bit
    return 0;
    }

int CI2C::Stop()
    {
    PORT_REG |= m_SDAW;
    PORT_REG &= ~m_SDAH; // SDA low
    I2C_DELAY();
    PORT_REG |= m_SCLH;        // Let SCL go up
    I2C_DELAY();
    PORT_REG |= m_SDAH; // then SDA
    I2C_DELAY();
    return 0;
    }


int CI2C::XBit(int bit)
   {
   PORT_REG |= m_SDAW;
   if (bit)
      {
      PORT_REG |= m_SDAH;
      }
   else
      {
      PORT_REG &= ~m_SDAH;
      }
   PORT_REG |= m_SCLH; //  SCL high
   PORT_REG &= ~m_SCLH; //  SCL low
   return 0;
   }

int CI2C::XBitSlow(int bit)
   {
   PORT_REG |= m_SDAW;
   if (bit)
      {
      PORT_REG |= m_SDAH;
      }
   else
      {
      PORT_REG &= ~m_SDAH;
      }
   I2C_DELAY();
   PORT_REG |= m_SCLH; //  SCL high
   I2C_DELAY();
   PORT_REG &= ~m_SCLH; //  SCL low
   I2C_DELAY();
   return 0;
   }


int CI2C::RBit()
   {
   int bit;
   PORT_REG &= ~m_SDAW;
   PORT_REG |= m_SCLH; //  SCL high
   bit = ((PORT_REG & m_SDAH) ? 1 : 0);             // Read in data
   PORT_REG &= ~m_SCLH; // SCL low
   return bit;                 // Return the sampled bit
   }

int CI2C::RBitSlow()
   {
   int bit;
   PORT_REG &= ~m_SDAW;
   PORT_REG |= m_SCLH; //  SCL high
   bit = ((PORT_REG & m_SDAH) ? 1 : 0);             // Read in data
   I2C_DELAY();
   PORT_REG &= ~m_SCLH; // SCL low
   I2C_DELAY();
   return bit;                 // Return the sampled bit
   }


int CI2C::XByte(u8 dat)
    {
    int bitCount;

    bitCount = 8;
    while(bitCount)
        {
        if (dat & 0x80)
            {
            XBit(1);
            }
        else
            {
            XBit(0);
            }
        dat <<= 1;
        bitCount--;
        }

    if (RBit())
        {
        m_error = I2CERR_NAK;
        return 1;
        }
    return 0;
    }


u8 CI2C::RByte(int ack)
    {
    u8 bitCount;
    u8 inByte;

    bitCount = 8;
    inByte = 0;

    while(bitCount)
        {
        inByte <<= 1;
        inByte |= RBit();
        bitCount--;
        }

    // acknowledge
    XBit(ack);
    return inByte;
    }


int CI2C::XmtBuf(u8 devId, u8 addr, int nBytes, u8* pBuf )
    {
    int cnt;

    m_error = 0;
    Start();
    if (XByte(devId))
        {
        return 1;
        }
    if (XByte(addr))
        {
        return 1;
        }
    for (cnt = 0; cnt < nBytes; cnt++)
        if (XByte(pBuf[cnt]))
            {
            return 1;
            }
    Stop();
    return 0;
    }




int CI2C::RcvBuf(u8 devId, u8 addr, int nBytes, u8* pBuf)
    {
    int cnt;

    m_error = 0;
    Start();
    if (XByte(devId))
        {
        return 1;
        }
    if (XByte(addr))
        {
        if (m_error == I2CERR_NAK) Stop();
        return 1;
        }
    PORT_REG |= m_SDAW;
    PORT_REG |= m_SDAH; // Set SDA high while SCL is low
    // Restart
    Start();
    if (XByte(devId|0x1))
        {
        return 1;
        }

    for (cnt = 0; cnt < nBytes-1; cnt++)
        {
        pBuf[cnt] = RByte(0);
        }
    // Master NAK
    pBuf[nBytes-1] = RByte(1);
    Stop();
    return (m_error ? 1 : 0);
    }



int CI2C::XmtByte(u8 devId, u8 addr, u8 data)
    {
    m_error = 0;
    Start();
    if (XByte(devId))
        {
        return 1;
        }
    if (XByte(addr))
        {
        return 1;
        }
    if (XByte(data))
        {
        return 1;
        }
    Stop();
    return 0;
    }




u8 CI2C::RcvByte(u8 devId, u8 addr)
    {
    u8 data;
    
    m_error = 0;
    Start();
    if (XByte(devId))
        {
        return 1;
        }
    if (XByte(addr))
        {
        if (m_error == I2CERR_NAK) Stop();
        return 1;
        }
    PORT_REG |= m_SDAW;
    PORT_REG |= m_SDAH; // Set SDA high while SCL is low
    // Restart
    Start();
    if (XByte(devId|0x1))
        {
        return 1;
        }
    data = RByte(1);
    Stop();
    return data;
    }

/*
 * $Header: $
 */
