#include "nvd.h"


int CNVD::BadChecksum()
    {
    int n;
    u8 cksum,cksumAct;
    cksum = 0;
    for (n = 0; n < NVD_DATA_BYTES; n++)
       {
       cksum += NVD_SRAM[n];
       }
    cksumAct = NVD_SRAM[NVD_DATA_BYTES];
    return ((cksum == cksumAct) ? 0 : 1);
    }

int CNVD::ReadPZTblSel()
   {
   int pzTblSel;
   pzTblSel = int(NVD_SRAM[0]);
   CLAMP(pzTblSel,SENSOR_MIN_PZTBL_SEL,SENSOR_MAX_PZTBL_SEL);
   return pzTblSel;
   }

int CNVD::ReadSinkThresholdCm()
   {
   int nSinkThresholdCm;
   nSinkThresholdCm = int(NVD_SRAM[1])*10;
   CLAMP(nSinkThresholdCm,VARIO_MIN_SINKTONE_THRESHOLD_CM,VARIO_MAX_SINKTONE_THRESHOLD_CM);
   return nSinkThresholdCm;
   }

int CNVD::ReadWindowSecs()
    {
    int nWindowSecs;
    nWindowSecs = int(NVD_SRAM[2]);
    CLAMP(nWindowSecs,SENSOR_MIN_WINDOW_SECS,SENSOR_MAX_WINDOW_SECS);
    return nWindowSecs;
    }

int CNVD::ReadCurInx()
   {
   int nCurInx;
   nCurInx = int(NVD_SRAM[3]);
   CLAMP(nCurInx,0,NVD_MAX_FLIGHTS-1);
   return nCurInx;
   }

int CNVD::ReadKLPFC()
   {
   int nKLPFC;
   nKLPFC = int(NVD_SRAM[4]);
   if (nKLPFC != SENSOR_KLPFC_H && nKLPFC != SENSOR_KLPFC_M && nKLPFC != SENSOR_KLPFC_L)
     nKLPFC = SENSOR_KLPFC_M;
   return nKLPFC;
   }

int CNVD::ReadVarioRes()
   {
   int nVarioRes;
   nVarioRes = int(NVD_SRAM[5]);
   if (nVarioRes != VARIO_RESOLUTION_H && nVarioRes != VARIO_RESOLUTION_M && nVarioRes != VARIO_RESOLUTION_L)
     nVarioRes = VARIO_RESOLUTION_M;
   return nVarioRes;
   }
   
int CNVD::ReadVarioAudioMode()
   {
   int nVarioMode;
   nVarioMode = int(NVD_SRAM[6]);
   if (nVarioMode != VARIO_MODE_CHIRP && nVarioMode != VARIO_MODE_VOICE)
     nVarioMode = VARIO_MODE_CHIRP;
   return nVarioMode;
   }

void CNVD::WriteDefaults()
   {
   int n;
   for (n = 0; n <= NVD_DATA_BYTES; n++)
      {
      NVD_SRAM[n] = 0;
      }
      
   WritePZTblSel(SENSOR_DFL_PZTBL_SEL);
   WriteWindowSecs(SENSOR_DFL_WINDOW_SECS);
   WriteKLPFC(SENSOR_DFL_KLPFC);
   WriteSinkThresholdCm(VARIO_DFL_SINKTONE_THRESHOLD_CM);
   WriteVarioRes(VARIO_DFL_RESOLUTION);
   WriteCurInx(0);
   WriteVarioAudioMode(VARIO_MODE_CHIRP);

   m_nCurInx = 0;
   for (n = 0; n < NVD_MAX_FLIGHTS; n++)
      ClearFlightRecord(n);
   }

void CNVD::ClearFlightRecord(int nrec)
   {
   int s,n;
   s = 8+18*nrec;
   n = 18;
   while (n--)
      NVD_SRAM[s++] = 0;
   }

void CNVD::CopyFlightRecord(int src, int dst)
   {
   int s,d,n;
   s = 8+18*src;
   d = 8+18*dst;
   n = 18;
   while (n--)
      NVD_SRAM[d++] = NVD_SRAM[s++];
   }
   
void CNVD::DeleteFlightRecord(int nrec)
   {
   int n;
   if (nrec == m_nCurInx)
      {
      ClearFlightRecord(nrec);
      m_nCurInx--;
      if (m_nCurInx < 0) m_nCurInx = NVD_MAX_FLIGHTS-1;
      WriteCurInx(m_nCurInx);
      }
   else
   if (nrec > m_nCurInx)
      {
      for (n = nrec-1; n > m_nCurInx; n--)
         CopyFlightRecord(n,n+1);
      ClearFlightRecord(m_nCurInx+1);
      }
   else
   if (m_nCurInx == NVD_MAX_FLIGHTS-1)
      {
      for (n = nrec-1; n >= 0; n--)
         CopyFlightRecord(n,n+1);
      ClearFlightRecord(0);
      }
   else
      {
      for (n = nrec-1; n >= 0; n--)
         CopyFlightRecord(n,n+1);
      CopyFlightRecord(NVD_MAX_FLIGHTS-1,0);
      for (n = NVD_MAX_FLIGHTS-2; n > m_nCurInx; n--)
         CopyFlightRecord(n,n+1);
      ClearFlightRecord(m_nCurInx+1);
      }
   UpdateChecksum();
   }
   
void CNVD::UpdateChecksum()
    {
    int n;
    u8 cksum;

    cksum = 0;
    for (n = 0; n < NVD_DATA_BYTES; n++)
        {
        cksum += NVD_SRAM[n];
        }
    NVD_SRAM[NVD_DATA_BYTES] = cksum;
    }


void CNVD::WriteByte(int addr, int newVal)
    {
    u8 cksum;

    cksum = NVD_SRAM[NVD_DATA_BYTES];
    cksum -= NVD_SRAM[addr];
    cksum += (u8)newVal;
    NVD_SRAM[addr] = (u8)newVal;
    NVD_SRAM[NVD_DATA_BYTES] = cksum;
    }

void CNVD::WriteWord(int addr, int newVal)
    {
    u8 cksum,b;

    cksum = NVD_SRAM[NVD_DATA_BYTES];
    cksum -= NVD_SRAM[addr];
    cksum -= NVD_SRAM[addr+1];
    b  = (u8)(newVal&0xFF);
    NVD_SRAM[addr] = b;
    cksum += b;
    b = (u8)((newVal>>8)&0xFF);
    NVD_SRAM[addr+1] = b;
    cksum += b;
    NVD_SRAM[NVD_DATA_BYTES] = cksum;
    }

void CNVD::WritePZTblSel(int pztblsel)
    {
    WriteByte(0,pztblsel);
    }

void CNVD::WriteSinkThresholdCm(int sinkthresh)
    {
    WriteByte(1,sinkthresh/10);
    }

void CNVD::WriteWindowSecs(int wsecs)
    {
    WriteByte(2,wsecs);
    }

void CNVD::WriteCurInx(int curinx)
    {
    WriteByte(3,curinx);
    }

void CNVD::WriteKLPFC(int klpfc)
    {
    WriteByte(4,klpfc);
    }

void CNVD::WriteVarioRes(int varioRes)
    {
    WriteByte(5,varioRes);
    }

void CNVD::WriteVarioAudioMode(int varioMode)
    {
    WriteByte(6,varioMode);
    }

void CNVD::WriteMaxCRCm(int inx, int maxcrcm)
    {
    WriteWord(8+18*inx+14, maxcrcm);
    }


void CNVD::WriteMaxSRCm(int inx, int maxsrcm)
    {
    WriteWord(8+18*inx+16, maxsrcm);
    }


void CNVD::WriteLaunchAltM(int inx, int altm)
    {
    WriteWord(8+18*inx+8, altm);
    }


void CNVD::WriteMaxAltM(int inx, int altm)
    {
    WriteWord(8+18*inx+10, altm);
    }
    
void CNVD::WriteFHrs(int inx, int fhrs)
    {
    WriteByte(8+18*inx+3, fhrs);
    }

void CNVD::WriteFMin(int inx, int fmin)
    {
    WriteByte(8+18*inx+4, fmin);
    }


void CNVD::WriteFSec(int inx, int fsec)
    {
    WriteByte(8+18*inx+5, fsec);
    }

void CNVD::WriteLMin(int inx, int lmin)
    {
    WriteByte(8+18*inx+7, lmin);
    }

void CNVD::WriteLHr(int inx, int lhr)
    {
    WriteByte(8+18*inx+6, lhr);
    }


void CNVD::WriteDom(int inx, int dom)
    {
    WriteByte(8+18*inx+2, dom-1);
    }


void CNVD::WriteMon(int inx, int mon)
    {
    WriteByte(8+18*inx+1, mon-1);
    }

void CNVD::WriteYear(int inx, int year)
    {
    WriteByte(8+18*inx, year);
    }


void CNVD::WriteLZAltM(int inx, int altm)
    {
    WriteWord(8+18*inx+12, altm);
    }

int CNVD::ReadWord(int addr)
   {
   int val;
   val = int(NVD_SRAM[addr]) | (int(NVD_SRAM[addr+1])<<8);
   return val;
   }

int CNVD::ReadLZAltM(int inx)
   {
   return ReadWord(8+18*inx+12);
   }

int CNVD::ReadLaunchAltM(int inx)
   {
   return ReadWord(8+18*inx+8);
   }

int CNVD::ReadMaxAltM(int inx)
   {
   return ReadWord(8+18*inx+10);
   }

int CNVD::ReadMaxCRCm(int inx)
   {
   return ReadWord(8+18*inx+14);
   }

int CNVD::ReadMaxSRCm(int inx)
   {
   return ReadWord(8+18*inx+16);
   }

int CNVD::ReadLHr(int inx)
   {
   return int(NVD_SRAM[8 +18*inx+6]);
   }

int CNVD::ReadLMin(int inx)
   {
   return int(NVD_SRAM[8 +18*inx+7]);
   }

int CNVD::ReadFSec(int inx)
   {
   return int(NVD_SRAM[8 +18*inx+5]);
   }


int CNVD::ReadFMin(int inx)
   {
   return int(NVD_SRAM[8 +18*inx+4]);
   }

int CNVD::ReadFHrs(int inx)
   {
   return int(NVD_SRAM[8 +18*inx+3]);
   }

int CNVD::ReadDom(int inx)
   {
   return int(NVD_SRAM[8 +18*inx+2]+1);
   }

int CNVD::ReadMon(int inx)
   {
   return int(NVD_SRAM[8 +18*inx+1]+1);
   }

int CNVD::ReadYear(int inx)
   {
   return int(NVD_SRAM[8 +18*inx]);
   }

/*
void CNVD::WriteFlightLog(int num)
    {
    int addr;

    addr = 8 + 18*num;
    NVD_SRAM[addr]   = m_nYar;
    NVD_SRAM[addr+1] = m_nMon;
    NVD_SRAM[addr+2] = m_nDom;
    NVD_SRAM[addr+3] = m_nFHrs;
    NVD_SRAM[addr+4] = m_nFMin;
    NVD_SRAM[addr+5] = m_nFSec;
    NVD_SRAM[addr+6] = m_nLHr;
    NVD_SRAM[addr+7] = m_nLMin;
    
    NVD_SRAM[addr+8] = (u8)(m_nLaunchAltM&0xFF);
    NVD_SRAM[addr+9] = (u8)(m_nLaunchAltM>>8);

    NVD_SRAM[addr+10] = (u8)(m_nMaxAltM&0xFF);
    NVD_SRAM[addr+11] = (u8)(m_nMaxAltM>>8);

    NVD_SRAM[addr+12] = (u8)(m_nLZAltM&0xFF);
    NVD_SRAM[addr+13] = (u8)(m_nLZAltM>>8);
    
    NVD_SRAM[addr+14] = (u8)(m_nMaxCRCm&0xFF);
    NVD_SRAM[addr+15] = (u8)(m_nMaxCRCm>>8);

    NVD_SRAM[addr+16] = (u8)(m_nMaxSRCm&0xFF);
    NVD_SRAM[addr+17] = (u8)(m_nMaxSRCm>>8);

    UpdateChecksum();
    }
*/


/*
 * $Header: $
 */

