#include "rtc.h"
#include "i2c.h"

#define ADDR_SEC    0
#define ADDR_MIN    1
#define ADDR_HRS    2
#define ADDR_DOW    3
#define ADDR_DOM    4
#define ADDR_MON    5
#define ADDR_YAR    6

extern CI2C gI2C;

void CRTC::Init()
   {
   u8 b;
   b = gI2C.RcvByte(I2C_ID_DS1338,0x0);
   // check CH bit is cleared to ensure oscillator is enabled
   if (b & 0x80)
      {
      gI2C.XmtByte(I2C_ID_DS1338,0x0,0);
      }
#ifdef SENSOR_SCP1000
   // disable square wave output
   gI2C.XmtByte(I2C_ID_DS1338,0x7,0);
#endif
#ifdef SENSOR_MS5534
   // enable 32.768kHz square wave output (sampling clock for MS5534B)
   gI2C.XmtByte(I2C_ID_DS1338,0x7,0x13);
#endif
#ifdef SENSOR_FOSP01
   // enable 32.768kHz square wave output (sampling clock for FOSP01)
   gI2C.XmtByte(I2C_ID_DS1338,0x7,0x13);
#endif
   }

void CRTC::ReadData()
   {
   u8 buf[7];
   gI2C.RcvBuf(I2C_ID_DS1338,0x0,7,buf);
   m_sec = (buf[0]&0xF) + 10*((buf[0]>>4)&7);
   CLAMP(m_sec,0,59);
   m_min = (buf[1]&0xF) + 10*((buf[1]>>4)&7);
   CLAMP(m_min,0,59);
   if (buf[2]&0x40)
      {
      m_hrsmode = RTC_HRS_12;
      m_hrs = (buf[2]&0xF) + 10*((buf[2]>>4)&1);
      CLAMP(m_hrs,0,12);
      m_ampm = (buf[2]>>5)&1;
      }
   else
      {
      m_hrsmode = RTC_HRS_24;
      m_hrs = (buf[2]&0xF) + 10*((buf[2]>>4)&3);
      CLAMP(m_hrs,0,23);
      m_ampm = m_hrs > 12 ? RTC_PM : RTC_AM;
      }

   m_dow = buf[3]&7;
   CLAMP(m_dow,1,7);
   m_dom = (buf[4]&0xF) + 10*((buf[4]>>4)&3);
   CLAMP(m_dom,1,31);
   m_mon = (buf[5]&0xF) + 10*((buf[5]>>4)&1);
   CLAMP(m_mon,1,12);
   m_yar = (buf[6]&0xF) + 10*(buf[6]>>4);
   CLAMP(m_yar,0,99);
   }

void CRTC::WriteData()
    {
    u8 buf[7];
    buf[0] = (u8)((m_sec%10) + ((m_sec/10)<<4));
    buf[1] = (u8)((m_min%10) + ((m_min/10)<<4));
    buf[2] = (u8)((m_hrs%10) + ((m_hrs/10)<<4) + (m_hrsmode<<5));
    buf[3] = (u8)m_dow;
    buf[4] = (u8)((m_dom%10) + ((m_dom/10)<<4));
    buf[5] = (u8)((m_mon%10) + ((m_mon/10)<<4));
    buf[6] = (u8)((m_yar%10) + ((m_yar/10)<<4));
    gI2C.XmtBuf(I2C_ID_DS1338,0x0,7,buf);
    }



void CRTC::IncrSec()
    {
    m_sec++;
    if (m_sec > 59) m_sec = 0;
    UpdateSec(m_sec);
    }
    
void CRTC::DecrSec()
    {
    m_sec--;
    if (m_sec < 0) m_sec = 59;
    UpdateSec(m_sec);
    }

void CRTC::UpdateSec(int sec)
    {
    u8 byte;
    byte = (u8)((sec%10) + ((sec/10)<<4));
    gI2C.XmtByte(I2C_ID_DS1338,ADDR_SEC,byte);
    }


void CRTC::IncrMin()
    {
    m_min++;
    if (m_min > 59) m_min = 0;
    UpdateMin(m_min);
    }

void CRTC::DecrMin()
    {
    m_min--;
    if (m_min < 0 ) m_min = 59;
    UpdateMin(m_min);
    }

void CRTC::UpdateMin(int min)
    {
    u8 byte;
    byte = (u8)((min%10) + ((min/10)<<4));
    gI2C.XmtByte(I2C_ID_DS1338,ADDR_MIN,byte);
    }

void CRTC::IncrHrs()
    {
    m_hrs++;
    if (m_hrsmode == RTC_HRS_12)
        {
        if (m_hrs > 12) m_hrs = 1;
        }
    else
        {
        if (m_hrs > 23) m_hrs = 0;
        }
    UpdateHrs(m_hrs,m_hrsmode);
    }

void CRTC::DecrHrs()
    {
    m_hrs--;
    if (m_hrsmode == RTC_HRS_12)
        {
        if (m_hrs < 1) m_hrs = 12;
        }
    else
        {
        if (m_hrs < 0) m_hrs = 23;
        }
    UpdateHrs(m_hrs,m_hrsmode);
    }

void CRTC::UpdateHrs(int hrs, int hrsmode)
    {
    u8 byte;
    byte = (u8)((hrs%10) + ((hrs/10)<<4) + (hrsmode<<5));
    gI2C.XmtByte(I2C_ID_DS1338,ADDR_HRS,byte);
    }

void CRTC::IncrDow()
    {
    m_dow++;
    if (m_dow > 7) m_dow = 1;
    UpdateDow(m_dow);
    }

void CRTC::DecrDow()
    {
    m_dow--;
    if (m_dow < 1) m_dow = 7;
    UpdateDow(m_dow);
    }

void CRTC::UpdateDow(int dow)
    {
    u8 byte;
    byte = (u8)dow;
    gI2C.XmtByte(I2C_ID_DS1338,ADDR_DOW,byte);
    }

void CRTC::IncrDom()
    {
    m_dom++;
    if (m_dom > 31) m_dom = 1;
    UpdateDom(m_dom);
    }

void CRTC::DecrDom()
    {
    m_dom--;
    if (m_dom < 1) m_dom = 31;
    UpdateDom(m_dom);
    }

void CRTC::UpdateDom(int dom)
    {
    u8 byte;
    byte = (u8)((dom%10) + ((dom/10)<<4));
    gI2C.XmtByte(I2C_ID_DS1338,ADDR_DOM,byte);
    }

void CRTC::IncrMon()
    {
    m_mon++;
    if (m_mon > 12) m_mon = 1;
    UpdateMon(m_mon);
    }

void CRTC::DecrMon()
    {
    m_mon--;
    if (m_mon < 1) m_mon = 12;
    UpdateMon(m_mon);
    }

void CRTC::UpdateMon(int mon)
    {
    u8 byte;
    byte = (u8)((mon%10) + ((mon/10)<<4));
    gI2C.XmtByte(I2C_ID_DS1338,ADDR_MON,byte);
    }

void CRTC::IncrYar()
    {
    m_yar++;
    if (m_yar > 99) m_yar = 0;
    UpdateYar(m_yar);
    }
    
void CRTC::DecrYar()
    {
    m_yar--;
    if (m_yar < 0) m_yar = 99;
    UpdateYar(m_yar);
    }
    
void CRTC::UpdateYar(int yar)
    {
    u8 byte;
    byte = (u8)((yar%10) + ((yar/10)<<4));
    gI2C.XmtByte(I2C_ID_DS1338,ADDR_YAR,byte);
    }

/*
 * $Header: $
 */
