#ifndef SENSOR_H_
#define SENSOR_H_

#include "def.h"

typedef struct SENSOR_SAMPLE_
   {
#ifdef USE_Z_REGRESSION
   int z;
#endif
   int p;
   int t;
   int tck;
   } SENSOR_SAMPLE;

#ifdef SENSOR_FOSP01
#include "fosp01.h"
#endif


#ifdef SENSOR_MS5534
#include "ms5534.h"
#endif

#ifdef SENSOR_SCP1000
#include "scp1000.h"
#endif

class CSensor
   {
   public :

   void Init();
   void InitData();
   void InitWindowBuffers();
   int  Pa2Cm(int pa, int pzInx, int tcInx);
   void LPFAltitude(int altCm);
   void LPFClimbRate(int cmPerSec);
   void ReadSampleContinuous();
   int  ClimbRateCmPerSec(int dPa, int Pa);
   void CalibrateT0();
   void GetAveragedAltitude(int nSamples);

#ifdef SENSOR_FOSP01
   void ReadCoeffs();
   void Reset();
   void TriggerPressureSample();
   void TriggerTemperatureSample();
   int  ReadPressure();
   int  ReadTemperature();
   void ConvertPressureTemperature(int pzInx, int tcInx);
   void AcquireAveragedSample(int nSamples, int pzInx, int tcInx);
   void AveragePressureTemperature();
   int  DeltaCmPerSec();
#endif

#ifdef SENSOR_MS5534
   void Reset();
   void ReadCoeffs();
   void TriggerPressureSample();
   void TriggerTemperatureSample();
   u16  ReadPressure();
   u16  ReadTemperature();
   void ConvertPressureTemperature(int pzInx, int tcInx);
   void AcquireAveragedSample(int nSamples, int pzInx, int tcInx);
#ifdef USE_P_REGRESSION
   int  DeltaPressurePerSec();
#endif
   void AveragePressureTemperature();
#ifdef USE_Z_REGRESSION
   int  DeltaCmPerSec();
#endif
#endif

#ifdef SENSOR_SCP1000
   void SetSampleMode(int port, int mode);
   void TriggerSample(int port);
   int  ReadTemperature(int port);
   int  ReadPressure(int port);
   int  AcquireAveragedSample(int port, int nSamples, int pzInx, int tcInx);
#ifdef USE_P_REGRESSION
   int  DeltaPressurePerSec(int port);
#endif
   void AveragePressureTemperature(int port);
   void InitWindowBuffers(int port);
#ifdef USE_Z_REGRESSION
   int  DeltaCmPerSec(int port);
#endif
#endif

   int  m_nCmPerSec;
   int  m_nCmPerSecx16;
   int  m_nAltM;
   int  m_nAltCm;
   int  m_nAltCmx16;
   int  m_nTempCx10;
#ifdef USE_P_REGRESSION
   f32  m_fTempC;
#endif
   int  m_nPresPa;
   int  m_nPZTblSel;
   int  m_nWindowSecs;
   int  m_nWindowSamples;
   int  m_nKLPFA;
   int  m_nKLPFC;
   int  m_nPresPaSample;
   int  m_nTempCx10Sample;
   int  m_nTempCorrInx;

#ifdef SENSOR_FOSP01
   SENSOR_SAMPLE m_SmpBuf[SENSOR_MAX_SAMPLES];
   int m_State;
   int m_nSmpCnt;
   int m_D1;
   int m_D2;
   int m_C1;
   int m_C2;
   int m_C3;
   int m_C4;
   int m_C5;
   int m_C6;
   int m_C7;
   int m_A;
   int m_B;
   int m_C;
   int m_D;
   u32 m_waitTick;
#endif

#ifdef SENSOR_MS5534
   SENSOR_SAMPLE m_SmpBuf[SENSOR_MAX_SAMPLES];
   int m_State;
   int m_nSmpCnt;
   u16 m_CoeffWord1;
   u16 m_CoeffWord2;
   u16 m_CoeffWord3;
   u16 m_CoeffWord4;
   u16 m_D1;
   u16 m_D2;

   int m_C1;
   int m_C2;
   int m_C3;
   int m_C4;
   int m_C5;
   int m_C6;
#endif

#ifdef SENSOR_SCP1000
   int m_nSampleMode;

   SENSOR_SAMPLE m_SmpBuf1[SENSOR_MAX_SAMPLES];
   SENSOR_SAMPLE m_SmpBuf2[SENSOR_MAX_SAMPLES];

   int m_nSmpCnt1;
   int m_nSmpCnt2;
   int m_nPresAvg1;
   int m_nTempAvg1;
   int m_nPresAvg2;
   int m_nTempAvg2;
#endif
   private :
   };


#endif

/*
 * $Header: $
 */

