#include "text.h"



void CText::Init(const u16* pPalette, const u8* pRawSource, char* pCharOrder, int wid, int hgt,int bScroll, int bx, int by, int adj,char* pTemplate,u8 objShape, u8 objSize)
    {
    m_pPalette = (u16*)pPalette;
    ham_LoadObjPal((void*)m_pPalette, 256);
    m_pRawSource = (u8*)pRawSource;
    m_pCharOrder = pCharOrder;
    m_wid = wid;
    m_hgt = hgt;
    m_widhgt   = m_wid*m_hgt;
    m_bScroll = bScroll;
    m_actCharPos = 0;
    m_bx = bx;
    m_by = by;
    m_adj = adj;
    SetText(pTemplate);
    CreateSprites(objShape, objSize);
    }

void CText::Init(const u16* pPalette, const u8* pRawSource, char* pCharOrder, int wid, int hgt,int bScroll, int bx, int by, int adj,int nChars,u8 objShape, u8 objSize)
    {
    m_pPalette = (u16*)pPalette;
    ham_LoadObjPal((void*)m_pPalette, 256);
    m_pRawSource = (u8*)pRawSource;
    m_pCharOrder = pCharOrder;
    m_wid = wid;
    m_hgt = hgt;
    m_widhgt   = m_wid*m_hgt;
    m_bScroll = bScroll;
    m_actCharPos = 0;
    m_bx = bx;
    m_by = by;
    m_adj = adj;
    SetText(nChars);
    CreateSprites(objShape, objSize);
    }

void CText::SetPos(int x, int y)
     {
     m_bx = x;
     m_by = y;
     }


 // CalcCharAddress(char Char)
 //
 // Desc: Calculates the offset of a char within the text data
 // Note: This means how many pixels are before the beginning of this char

inline int CText::CalcCharAddress(char c)
    {
    int pos=0;
    char *pChar = &(m_pCharOrder[0]);
    
    while(*pChar)
        {
        if(c == *pChar++) break;
        pos++;
        }

    return (m_widhgt * pos);
    }


inline int CText::StrLen(char* pText)
    {
    int len = 0;
    while (pText[len] != '\0') len++;
    return len;
    }
    
    
void CText::SetText(char *pText)
    {
    int n;
    m_strLen = StrLen(pText);
    n = 0;
    while(n < m_strLen)
        {
        m_text[n] = pText[n];
        n++;
        }
    while(n < MAX_FIELD_CHARS)
        {
        m_text[n] = '\0';
        n++;
        }
    }

void CText::SetText(int nChars)
    {
    int n;
    m_strLen = nChars;
    n = 0;
    while(n < m_strLen)
        {
        m_text[n] = ' ';
        n++;
        }
    while(n < MAX_FIELD_CHARS)
        {
        m_text[n] = '\0';
        n++;
        }
    }



void CText::CreateSprites(u8 objShape, u8 objSize)
    {
    int nChars,c;
    
    nChars = m_bScroll ? SCROLL_NCHARS(m_wid,m_adj) : m_strLen;
    for (c = 0; c < nChars; c++)
        {
        m_spriteInfo[c].x = m_bx  + c*(m_wid-m_adj);
        m_spriteInfo[c].y = m_by;
        m_spriteInfo[c].inx =
        ham_CreateObj((void*)&(m_pRawSource[CalcCharAddress(m_text[c])]),
            objShape,objSize,OBJ_MODE_NORMAL,1,0,1,0,0,0,0,m_spriteInfo[c].x,m_spriteInfo[c].y);
        }
    m_actCharPos = m_bScroll ? nChars : 0;
    }


void CText::DeleteSprites(void)
    {
    int nChars,c;

    nChars = m_bScroll ? SCROLL_NCHARS(m_wid,m_adj) : m_strLen;
    for (c = 0; c < nChars; c++)
        {
        ham_DeleteObj(m_spriteInfo[c].inx);
        }
    }


void CText::DisplayScrollR2L()
    {
    int c,nChars;
    nChars = SCROLL_NCHARS(m_wid,m_adj);
    for (c = 0; c < nChars; c++)
        {
        if (m_spriteInfo[c].x + (m_wid-m_adj) < 0)
            {
            ham_UpdateObjGfx(m_spriteInfo[c].inx, (void*)&(m_pRawSource[CalcCharAddress(m_text[m_actCharPos])]));
            m_actCharPos++;
            if (m_actCharPos > m_strLen)
                m_actCharPos = 0;
            m_spriteInfo[c].x = (nChars-1)*(m_wid-m_adj);
            }
        m_spriteInfo[c].x--;
        ham_SetObjXY(m_spriteInfo[c].inx, m_spriteInfo[c].x,  m_by);
        }
    }

void CText::DisplayStatic()
    {
    int c;

    for (c = 0; c < m_strLen; c++)
        {
        ham_UpdateObjGfx(m_spriteInfo[c].inx, (void*)&(m_pRawSource[CalcCharAddress(m_text[c])]));
        m_spriteInfo[c].x = m_bx + c*(m_wid-m_adj);
        m_spriteInfo[c].y = m_by;
        ham_SetObjXY(m_spriteInfo[c].inx, m_spriteInfo[c].x, m_spriteInfo[c].y);
        }
    }
    
    
void CText::Clear()
    {
    int c,nChars;
    nChars = m_bScroll ? SCROLL_NCHARS(m_wid,m_adj) : m_strLen;
    for (c = 0; c < nChars; c++)
        {
        ham_UpdateObjGfx(m_spriteInfo[c].inx, (void*)&(m_pRawSource[CalcCharAddress(' ')]));
        }
     }
     
/*
 * $Header: $
 */

