#include <hel2.h>
#include "def.h"
#include "utils.h"
#include "nvd.h"
#include "i2c.h"
#include "text.h"
#include "vario.h"
#include "rtc.h"
#include "sensor.h"
#include "tmr.h"
#include "ui.h"
#include "voice.h"

extern CSensor gSensor;
extern CVario  gVario;
extern CNVD    gNVD;
extern CTimer  gTimer;
extern CRTC    gRTC;
extern CI2C    gI2C;
extern CVoice  gVoice;
extern FLIGHT_DATA gFD;

extern int gnMode;
extern int gnFrameCnt;

const char szMon[12][4] = {"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
const char szDay[7][4] = {"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};

char szBuf[15] = "";

CText   gTxtAlt;
CText   gTxtAltAL;
CText   gTxtTemp;
CText   gTxtFTime;
CText   gTxtRTime;
CText   gTxtFilter;
CText   gTxtVRes;
CText   gTxtLArrow;
CText   gTxtRArrow;

static int gnSimClimbRate;
static int gnSimAltALCm;
int gnLogRec;
int gnParSel;
volatile int gbVBlank;


#include "gfx/vario_mgd.map.c"
#include "gfx/vario_mgd.raw.c"
#include "gfx/vario_mgd.pal.c"

#include "gfx/font.c"


void SimEventHandler(void)
   {
   int nFrmSlot;
   static int keyDelta;

	if ((gTmr.runFlag == false) && (ABS(gnSimAltALCm/100) > NVD_ATO_THRESHOLD)) 
      {
		gTmr.runFlag = true;
	   }

   if (gVario.m_nMode == VARIO_MODE_CHIRP && gTmr.varioTickFlag == true)
      {
      gTmr.varioTickFlag = false;
      gVario.Chirp(gnSimClimbRate);
      }

   if (gTmr.secFlag == true) 
      {
		gTmr.secFlag = false;
		gnSimAltALCm += gnSimClimbRate;
	   }

   if (gbVBlank == true)
      {
      if (gVario.m_nMode == VARIO_MODE_VOICE)
         {
         ham_UpdateMixer();
         gVario.Voice(gnSimClimbRate);
         }
      gbVBlank = false;
      gnFrameCnt++;
      ham_UpdatePad();
      DisplaySlider(gnSimClimbRate);

      if (Pad.Pressed.Start)
         {
         if (gVario.m_nMode == VARIO_MODE_CHIRP)
            {
            VARIO_SOUND_RESET();
            }
         else
            {
            M_SNDCNT2_DSOUNDA_OUTPUT_R_ENABLE(0);
            M_SNDCNT2_DSOUNDA_OUTPUT_L_ENABLE(0);
            ham_KillSample(gVoice.m_wave[gVoice.m_nVoiceSel]);
            ham_UpdateMixer();
            gVoice.m_nVoiceSel = -1;
            }
         DeleteSprites();
         ResetBkg();
         gSensor.m_nCmPerSec = 0;
         gVario.m_nCmPerSec = 0;
         InitLogMode();
         }
      else
      if (Pad.Pressed.R)
         {
         gVario.SetMode(gVario.m_nMode == VARIO_MODE_CHIRP ? VARIO_MODE_VOICE : VARIO_MODE_CHIRP);
         }
      else
      if (Pad.Held.Down && gnSimClimbRate >= (-VARIO_MAX_CLIMB_RATE_DPY_CM+keyDelta))
         {
         if (gnFrameCnt % 3 == 0)
            {
            gnSimClimbRate -= keyDelta++;
            CLAMP(keyDelta,0,5);
            }
         }
      else
      if (Pad.Held.Up && gnSimClimbRate <= (VARIO_MAX_CLIMB_RATE_DPY_CM-keyDelta))
         {
         if (gnFrameCnt % 3 == 0)
            {
            gnSimClimbRate += keyDelta++;
            CLAMP(keyDelta,0,5);
            }
         }
      else
         {
         keyDelta = 1;
         }


       nFrmSlot = gnFrameCnt % 64;
       switch (nFrmSlot)
          {
          case 0 :
  			 PrintAlt(szBuf,SIM_START_ALTITUDE_M + gnSimAltALCm/100);
          gTxtAlt.SetText(szBuf);
          gTxtAlt.DisplayStatic();
          break;

          case 2 :
			 PrintAltAL(szBuf, gnSimAltALCm/100);
			 gTxtAltAL.SetText(szBuf);
			 gTxtAltAL.DisplayStatic();
			 break;

          case 4 :
          PrintTemperature(szBuf,SIM_START_TEMP_CX10 - (gnSimAltALCm*65)/100000);
          gTxtTemp.SetText(szBuf);
          gTxtTemp.DisplayStatic();
          break;

          case 6 :
          PrintRealTime(szBuf);
          gTxtRTime.SetText(szBuf);
          gTxtRTime.DisplayStatic();
          break;

          case 8 :
   		 if (gTmr.runFlag == true) 
             {
				 StrPrintf(szBuf,"%02d:%02d:%02d", gTmr.fhrs, gTmr.fmins, gTmr.fsecs);
				 gTxtFTime.SetText(szBuf);
				 gTxtFTime.DisplayStatic();
				 }
          break;

          default :
          break;
          }
       }
    }



void RunEventHandler(void)
   {
   int nFrmSlot;
   u32 sTick;

   if ((gTmr.runFlag == false) && (ABS(gSensor.m_nAltM - gFD.nLaunchAltM) > NVD_ATO_THRESHOLD))
      {
      gTmr.runFlag = true;
      gNVD.WriteLHr(gNVD.m_nCurInx,gTmr.rhrs);
      gNVD.WriteLMin(gNVD.m_nCurInx,gTmr.rmins);
      }

   sTick = gTmr.ticks;
   gSensor.ReadSampleContinuous();

   if (gVario.m_nMode == VARIO_MODE_CHIRP && gTmr.varioTickFlag == true)
      {
      gTmr.varioTickFlag = false;
      gVario.Chirp(gSensor.m_nCmPerSec);
      }

   if (gTmr.runFlag == true) 
      {
      if (gFD.nMaxAltM < gSensor.m_nAltM)
         gFD.nMaxAltM = gSensor.m_nAltM;
      if (gFD.nMaxCRCm < gSensor.m_nCmPerSec)
         gFD.nMaxCRCm = gSensor.m_nCmPerSec;
      if (gFD.nMaxSRCm < -gSensor.m_nCmPerSec)
         gFD.nMaxSRCm = -gSensor.m_nCmPerSec;

      if (gTmr.secFlag == true)
         {
         gTmr.secFlag = false;
         StrPrintf(szBuf,"%02d:%02d:%02d",gTmr.fhrs,gTmr.fmins,gTmr.fsecs);
         gTxtFTime.SetText(szBuf);
         gTxtFTime.DisplayStatic();
         gNVD.WriteFHrs(gNVD.m_nCurInx,gTmr.fhrs);
         gNVD.WriteFMin(gNVD.m_nCurInx,gTmr.fmins);
         gNVD.WriteFSec(gNVD.m_nCurInx,gTmr.fsecs);
         }
      }

   if (gbVBlank == true)
      {
      if (gVario.m_nMode == VARIO_MODE_VOICE)
         {
         ham_UpdateMixer();
         gVario.Voice(gSensor.m_nCmPerSec);
         }
      gbVBlank = false;
      gnFrameCnt++;
      ham_UpdatePad();

      DisplaySlider(gSensor.m_nCmPerSec);
      if (Pad.Pressed.Select)
         {
         ResetLogRecord();
         gNVD.WriteLHr(gNVD.m_nCurInx,gTmr.rhrs);
         gNVD.WriteLMin(gNVD.m_nCurInx,gTmr.rmins);
         }
      else
      if (Pad.Pressed.B)
         {
         if (gSensor.m_nKLPFC == SENSOR_KLPFC_L)
            {
            gSensor.m_nKLPFC = SENSOR_KLPFC_M;
            gTxtFilter.SetText("*");
            }
         else
         if (gSensor.m_nKLPFC == SENSOR_KLPFC_M)
            {
            gSensor.m_nKLPFC = SENSOR_KLPFC_H;
            gTxtFilter.SetText("]");
            }
         else
            {
            gSensor.m_nKLPFC = SENSOR_KLPFC_L;
            gTxtFilter.SetText("[");
            }
         gNVD.WriteKLPFC(gSensor.m_nKLPFC);
         gTxtFilter.DisplayStatic();
         }
      else
      if (Pad.Pressed.A && (gVario.m_nMode == VARIO_MODE_CHIRP))
         {
         if (gVario.m_nResolutionCm == VARIO_RESOLUTION_L)
            {
            gVario.m_nResolutionCm = VARIO_RESOLUTION_M;
            gTxtVRes.SetText("#");
            }
         else
         if (gVario.m_nResolutionCm == VARIO_RESOLUTION_M)
            {
            gVario.m_nResolutionCm = VARIO_RESOLUTION_H;
            gTxtVRes.SetText(">");
            }
         else
            {
            gVario.m_nResolutionCm = VARIO_RESOLUTION_L;
            gTxtVRes.SetText("<");
            }
         gNVD.WriteVarioRes(gVario.m_nResolutionCm);
         gTxtVRes.DisplayStatic();
         }
      else
      if (Pad.Pressed.Start) // GO TO LOG MODE
         {
         if (gVario.m_nMode == VARIO_MODE_CHIRP)
            {
            VARIO_SOUND_RESET();
            }
         else
            {
            M_SNDCNT2_DSOUNDA_OUTPUT_R_ENABLE(0);
            M_SNDCNT2_DSOUNDA_OUTPUT_L_ENABLE(0);
            if (gVoice.m_wave[gVoice.m_nVoiceSel]->playing)
               {
               ham_KillSample(gVoice.m_wave[gVoice.m_nVoiceSel]);
               ham_UpdateMixer();
               }
            gVoice.m_nVoiceSel = -1;
            }
         DeleteSprites();
         ResetBkg();
         if (gTmr.fhrs == 0 && gTmr.fmins == 0 && gTmr.fsecs == 0)
            {
            gNVD.DeleteFlightRecord(gNVD.m_nCurInx);
            }
         gSensor.m_nCmPerSec = 0;
         gVario.m_nCmPerSec = 0;
         InitLogMode();
         }
      else
      if (Pad.Pressed.R)
         {
         gVario.SetMode(gVario.m_nMode == VARIO_MODE_CHIRP ? VARIO_MODE_VOICE : VARIO_MODE_CHIRP);
         if (gVario.m_nMode == VARIO_MODE_VOICE)
            {
            gTxtVRes.SetText(" ");
            }
         else
            {
            if (gVario.m_nResolutionCm == VARIO_RESOLUTION_L)
               {
               gTxtVRes.SetText("<");
               }
            else
            if (gVario.m_nResolutionCm == VARIO_RESOLUTION_M)
               {
               gTxtVRes.SetText("#");
               }
            else
               {
               gTxtVRes.SetText(">");
               }
            }
         gTxtVRes.DisplayStatic();
         }

         
      nFrmSlot = gnFrameCnt % 64;
      switch (nFrmSlot)
         {
         case 0 :
         PrintAlt(szBuf,gSensor.m_nAltM);
         gTxtAlt.SetText(szBuf);
         gTxtAlt.DisplayStatic();
         break;


         case 2 :
         PrintAltAL(szBuf,gSensor.m_nAltM - gFD.nLaunchAltM);
         gTxtAltAL.SetText(szBuf);
         gTxtAltAL.DisplayStatic();
         break;

         case 4 :
         PrintTemperature(szBuf,gSensor.m_nTempCx10);
         gTxtTemp.SetText(szBuf);
         gTxtTemp.DisplayStatic();
         break;

         case 6 :
         PrintRealTime(szBuf);
         gTxtRTime.SetText(szBuf);
         gTxtRTime.DisplayStatic();
        // ham_DrawText(0,3,"RTM%03d",(gTmr.ticks - stick)*5);
         break;

         case 8 :
         if (gTmr.runFlag == true)
            {
            gNVD.WriteMaxAltM(gNVD.m_nCurInx,gFD.nMaxAltM);
            gNVD.WriteLZAltM(gNVD.m_nCurInx,gSensor.m_nAltM);
         //   ham_DrawText(0,4,"NVD%03d",(gTmr.ticks - stick)*5);
            }
         break;

         case 10 :
         if (gTmr.runFlag == true)
            {
            gNVD.WriteMaxCRCm(gNVD.m_nCurInx,gFD.nMaxCRCm);
            gNVD.WriteMaxSRCm(gNVD.m_nCurInx,gFD.nMaxSRCm);
            }
         break;

         default :
         break;
         }
      }
   }



void LogEventHandler(void)
   {
   int ix;
   
   if (gbVBlank == true)
      {
      gbVBlank = false;
      gnFrameCnt++;
      if ((gnFrameCnt & 63) == 0)
         DisplayDate();

      ham_UpdatePad();

      if (Pad.Pressed.Left)
         {
         gnLogRec =  (gnLogRec > 0)? gnLogRec-1 : NVD_MAX_FLIGHTS-1;
         DisplayLogRecord(gnLogRec);
         }
      else
      if (Pad.Pressed.Right)
         {
         gnLogRec = (gnLogRec < NVD_MAX_FLIGHTS-1)?  gnLogRec+1 : 0;
         DisplayLogRecord(gnLogRec);
         }
      else
      if (Pad.Pressed.Select)
         {
         ix = gNVD.m_nCurInx - gnLogRec;
         if (ix < 0)
            ix += NVD_MAX_FLIGHTS;
         gNVD.DeleteFlightRecord(ix);
         DisplayLogRecord(gnLogRec);
         }
      else
      if (Pad.Pressed.Start)
         {
         InitSetMode();
         }
      }
  }


void SetEventHandler(void)
   {
   if (gbVBlank == true)
      {
      gbVBlank = false;
      gnFrameCnt++;
      ham_UpdatePad();
      if (Pad.Pressed.Up)
         {
         gnParSel = gnParSel ? gnParSel-1 : SEL_RTC_MIN;
         DisplaySettings();
         }
      else
      if (Pad.Pressed.Down)
         {
         gnParSel = gnParSel < SEL_RTC_MIN ? gnParSel+1 : SEL_ALTITUDE;
         DisplaySettings();
         }
      else
      if (Pad.Pressed.Left)
         {
         switch (gnParSel)
            {
            case SEL_ALTITUDE :
            default :
            if (gSensor.m_nPZTblSel > SENSOR_MIN_PZTBL_SEL)
               {
               gSensor.m_nPZTblSel--;
               gNVD.WritePZTblSel(gSensor.m_nPZTblSel);
               gSensor.GetAveragedAltitude(4);
               }
            break;

            case SEL_WINDOW_SECS :
            if (gSensor.m_nWindowSecs > SENSOR_MIN_WINDOW_SECS)
               {
               gSensor.m_nWindowSecs--;
               gSensor.m_nWindowSamples = gSensor.m_nWindowSecs * SENSOR_SAMPLES_PER_SEC;
               gNVD.WriteWindowSecs(gSensor.m_nWindowSecs);
               }
            break;

            case SEL_VARIO_AUDIO :
            gVario.SetMode(VARIO_MODE_CHIRP);
            gNVD.WriteVarioAudioMode(VARIO_MODE_CHIRP);
            break;

            case SEL_SINK_THRESH :
            if (gVario.m_nSinkThresholdCm >= (VARIO_MIN_SINKTONE_THRESHOLD_CM+50))
               {
               gVario.m_nSinkThresholdCm -= 50;
               gNVD.WriteSinkThresholdCm(gVario.m_nSinkThresholdCm);
               }
            break;

            case SEL_RTC_YAR :
               gRTC.DecrYar();
            break;

            case SEL_RTC_MON :
               gRTC.DecrMon();
            break;

            case SEL_RTC_DOM :
               gRTC.DecrDom();
            break;

            case SEL_RTC_DOW :
               gRTC.DecrDow();
               gTimer.SetRealTime(gRTC.m_dow,gRTC.m_hrs,gRTC.m_min,0);
            break;

            case SEL_RTC_HRS :
               gRTC.DecrHrs();
               gTimer.SetRealTime(gRTC.m_dow,gRTC.m_hrs,gRTC.m_min,0);
            break;

            case SEL_RTC_MIN :
               gRTC.DecrMin();
               gTimer.SetRealTime(gRTC.m_dow,gRTC.m_hrs,gRTC.m_min,0);
            break;
            }
         DisplaySettings();
         }
      else
      if (Pad.Pressed.Right)
         {
         switch (gnParSel)
            {
            case SEL_ALTITUDE :
            default :
            if (gSensor.m_nPZTblSel < SENSOR_MAX_PZTBL_SEL)
               {
               gSensor.m_nPZTblSel++;
               gNVD.WritePZTblSel(gSensor.m_nPZTblSel);
               gSensor.GetAveragedAltitude(4);
               }
            break;

            case SEL_WINDOW_SECS :
            if (gSensor.m_nWindowSecs < SENSOR_MAX_WINDOW_SECS)
               {
               gSensor.m_nWindowSecs++;
               gSensor.m_nWindowSamples = gSensor.m_nWindowSecs*SENSOR_SAMPLES_PER_SEC;
               gNVD.WriteWindowSecs(gSensor.m_nWindowSecs);
               }
            break;

            case SEL_VARIO_AUDIO :
            gVario.SetMode(VARIO_MODE_VOICE);
            gNVD.WriteVarioAudioMode(VARIO_MODE_VOICE);
            break;

            case SEL_SINK_THRESH :
            if (gVario.m_nSinkThresholdCm <= (VARIO_MAX_SINKTONE_THRESHOLD_CM-50))
               {
               gVario.m_nSinkThresholdCm += 50;
               gNVD.WriteSinkThresholdCm(gVario.m_nSinkThresholdCm);
               }
            break;

            case SEL_RTC_YAR :
               gRTC.IncrYar();
            break;

            case SEL_RTC_MON :
               gRTC.IncrMon();
            break;

            case SEL_RTC_DOM :
               gRTC.IncrDom();
            break;

            case SEL_RTC_DOW :
               gRTC.IncrDow();
               gTimer.SetRealTime(gRTC.m_dow,gRTC.m_hrs,gRTC.m_min,0);
            break;

            case SEL_RTC_HRS :
               gRTC.IncrHrs();
               gTimer.SetRealTime(gRTC.m_dow,gRTC.m_hrs,gRTC.m_min,0);
            break;

            case SEL_RTC_MIN :
               gRTC.IncrMin();
               gTimer.SetRealTime(gRTC.m_dow,gRTC.m_hrs,gRTC.m_min,0);
            break;
            }
         DisplaySettings();
         }
      else
      if (Pad.Pressed.A)
         {
         gSensor.m_nPZTblSel = SENSOR_DFL_PZTBL_SEL;
         gSensor.m_nTempCorrInx = SENSOR_DFL_TEMP_CORR_INX;
         gNVD.WritePZTblSel(gSensor.m_nPZTblSel);
         gSensor.GetAveragedAltitude(4);
         DisplaySettings();
         }
      else
      if (Pad.Pressed.Start)
         {
         InitRunMode();
         }
      else
      if (Pad.Pressed.L)
         {
         InitSimMode();
         }

      if ((gnFrameCnt & 63) == 0)
         {
         DisplayDate();
         }
      }
   }

   
void InitLogMode()
   {
   gnMode = MODE_LOG;
   gnLogRec = 0;
   DisplayLogRecord(gnLogRec);
   }

void InitSetMode()
   {
   gnMode = MODE_SETTINGS;
   gnParSel = SEL_ALTITUDE;
   gSensor.CalibrateT0();
   ClearText(20);
   DisplaySettings();
   }

void InitSimMode()
   {
   gnMode = MODE_SIMULATION;
   ClearText(20);
   ResetScreen();
   ham_SetBgPalColRGB(0,255,255,00);
   gTxtLArrow.SetText("l");
   gTxtLArrow.DisplayStatic();
   gTxtRArrow.SetText("r");
   gTxtRArrow.DisplayStatic();
   PrintAlt(szBuf,SIM_START_ALTITUDE_M);
   gTxtAlt.SetText(szBuf);
   gTxtAlt.DisplayStatic();
   PrintTemperature(szBuf,SIM_START_TEMP_CX10);
   gTxtTemp.SetText(szBuf);
   gTxtTemp.DisplayStatic();
   PrintAltAL(szBuf,0);
   gTxtAltAL.SetText(szBuf);
   gTxtAltAL.DisplayStatic();
   gTxtFTime.SetText("--:--:--");
   gTxtFTime.DisplayStatic();
   gTxtVRes.SetText("-");
   gTxtVRes.DisplayStatic();
   gTxtFilter.SetText("-");
   gTxtFilter.DisplayStatic();

   gnSimClimbRate = 0;
   gnSimAltALCm = 0;
   gVario.m_nCmPerSec = 0;
   gTimer.Reset();
   if (gVario.m_nMode == VARIO_MODE_VOICE)
      {
      M_SNDCNT2_DSOUNDA_OUTPUT_R_ENABLE(1);
      M_SNDCNT2_DSOUNDA_OUTPUT_L_ENABLE(1);
      }
   else
      {
      M_SNDCNT1_OUTPUT_LEFT_ALL_ON;
      M_SNDCNT1_OUTPUT_RIGHT_ALL_ON;
      }
   }


void InitRunMode()
   {
   gnMode = MODE_RUNNING;
   gVario.m_nCmPerSec = 0;
 
   gSensor.GetAveragedAltitude(4);

   gSensor.InitData();
#ifdef SENSOR_SCP1000
   gSensor.SetSampleMode(I2C_PORT1,SENSOR_SAMPLE_CONTINUOUS);
   BusyWait(60);
   gSensor.SetSampleMode(I2C_PORT2,SENSOR_SAMPLE_CONTINUOUS);
#endif
   ResetScreen();
   ClearText(20);
   gTxtLArrow.SetText("l");
   gTxtLArrow.DisplayStatic();
   gTxtRArrow.SetText("r");
   gTxtRArrow.DisplayStatic();
   PrintAlt(szBuf,gSensor.m_nAltM);
   gTxtAlt.SetText(szBuf);
   gTxtAlt.DisplayStatic();
   PrintTemperature(szBuf,gSensor.m_nTempCx10);
   gTxtTemp.SetText(szBuf);
   gTxtTemp.DisplayStatic();
   PrintAltAL(szBuf,0);
   gTxtAltAL.SetText(szBuf);
   gTxtAltAL.DisplayStatic();
   PrintRealTime(szBuf);
   gTxtRTime.SetText(szBuf);
   gTxtRTime.DisplayStatic();
   gTxtFTime.SetText("--:--:--");
   gTxtFTime.DisplayStatic();

   if (gVario.m_nMode == VARIO_MODE_VOICE)
      {
      gTxtVRes.SetText(" ");
      }   
   else
      {
      if (gVario.m_nResolutionCm == VARIO_RESOLUTION_H)
         gTxtVRes.SetText(">");
      else
      if (gVario.m_nResolutionCm == VARIO_RESOLUTION_M)
         gTxtVRes.SetText("#");
      else
         gTxtVRes.SetText("<");
      }
   gTxtVRes.DisplayStatic();
   
   if (gSensor.m_nKLPFC == SENSOR_KLPFC_H)
      gTxtFilter.SetText("]");
   else
   if (gSensor.m_nKLPFC == SENSOR_KLPFC_M)
      gTxtFilter.SetText("*");
   else
      gTxtFilter.SetText("[");
   gTxtFilter.DisplayStatic();

   gNVD.m_nCurInx++;
   if (gNVD.m_nCurInx == NVD_MAX_FLIGHTS)
      gNVD.m_nCurInx = 0;
   gNVD.WriteCurInx(gNVD.m_nCurInx);
   ResetLogRecord();
   gTimer.Reset();
   if (gVario.m_nMode == VARIO_MODE_VOICE)
      {
      M_SNDCNT2_DSOUNDA_OUTPUT_R_ENABLE(1);
      M_SNDCNT2_DSOUNDA_OUTPUT_L_ENABLE(1);
      }
   else
      {
      M_SNDCNT1_OUTPUT_LEFT_ALL_ON;
      M_SNDCNT1_OUTPUT_RIGHT_ALL_ON;
      }
   }

void DisplaySettings()
    {
    int n;

    ham_DrawText(10, 1,"SETTINGS");
    for (n = 0; n <= SEL_RTC_MIN; n++)
       ham_DrawText(1,SEL_Y_ORIG+n,"%s",gnParSel == n ? "x " : "  ");

    n = 0;
    ham_DrawText(3, SEL_Y_ORIG+n, "Altitude          : %04dm",gSensor.m_nAltM);n++;
    ham_DrawText(3, SEL_Y_ORIG+n, "Averaging Window  : %ds",gSensor.m_nWindowSecs);n++;
    ham_DrawText(3, SEL_Y_ORIG+n, "Vario Audio       : %s",gVario.m_nMode == VARIO_MODE_VOICE ? "Voice" : "Chirp");n++;
    ham_DrawText(3, SEL_Y_ORIG+n, "Sinktone Threshold: %d.%dm/s",gVario.m_nSinkThresholdCm/100,(gVario.m_nSinkThresholdCm%100)/10);n++;
    ham_DrawText(3, SEL_Y_ORIG+n, "Year              : %d",2000+gRTC.m_yar);n++;
    ham_DrawText(3, SEL_Y_ORIG+n, "Month             : %s",szMon[gRTC.m_mon-1]);n++;
    ham_DrawText(3, SEL_Y_ORIG+n, "Day of Month      : %02d",gRTC.m_dom);n++;
    ham_DrawText(3, SEL_Y_ORIG+n, "Day of Week       : %s",szDay[gRTC.m_dow-1]);n++;
    ham_DrawText(3, SEL_Y_ORIG+n, "Hour              : %02d",gRTC.m_hrs);n++;
    ham_DrawText(3, SEL_Y_ORIG+n, "Minute            : %02d",gRTC.m_min);n++;
    }


void DisplayLogRecord(int nrec)
   {
   int ix,row,lhr;

   ClearText(20);
   DisplayDate();
   ham_DrawText(10,1,"Flight %d", nrec+1);
   ix = gNVD.m_nCurInx - nrec;
   if (ix < 0)
      ix += NVD_MAX_FLIGHTS;

   row = 0;
   ham_DrawText(LOG_X,LOG_Y+row,"Date          20%02d %s %02d",gNVD.ReadYear(ix), szMon[gNVD.ReadMon(ix)-1], gNVD.ReadDom(ix));
   row++;

   lhr = gNVD.ReadLHr(ix);
   if (lhr/12)
      {
      ham_DrawText(LOG_X,LOG_Y+row,"Launch        %02d:%02dPM",(lhr-12) ? (lhr-12) : 12,gNVD.ReadLMin(ix));
      }
   else
      {
      ham_DrawText(LOG_X,LOG_Y+row,"Launch        %02d:%02dAM",lhr,gNVD.ReadLMin(ix));
      }
   row++;

   ham_DrawText(LOG_X,LOG_Y+row,"Duration      %02d:%02d:%02d",gNVD.ReadFHrs(ix),gNVD.ReadFMin(ix), gNVD.ReadFSec(ix));
   row++;

   PrintAlt(szBuf,gNVD.ReadLaunchAltM(ix));
   ham_DrawText(LOG_X,LOG_Y+row,"Launch Alt    %s",szBuf);row++;

   PrintAlt(szBuf,gNVD.ReadMaxAltM(ix));
   ham_DrawText(LOG_X,LOG_Y+row,"Maximum Alt   %s",szBuf);row++;

   PrintAlt(szBuf,gNVD.ReadLZAltM(ix));
   ham_DrawText(LOG_X,LOG_Y+row,"LZ Alt        %s",szBuf);row++;

   PrintClimbRate(szBuf,gNVD.ReadMaxCRCm(ix));
   ham_DrawText(LOG_X,LOG_Y+row,"Max Climb    %s",szBuf);row++;

   PrintClimbRate(szBuf,gNVD.ReadMaxSRCm(ix));
   ham_DrawText(LOG_X,LOG_Y+row,"Max Sink     %s",szBuf);row++;
   }


void ResetLogRecord()
   {
   gFD.nMaxCRCm = 0;
   gFD.nMaxSRCm  = 0;

   gFD.nMaxAltM = gFD.nLZAltM = gFD.nLaunchAltM = gSensor.m_nAltM;
   gNVD.WriteYear(gNVD.m_nCurInx,gRTC.m_yar);
   gNVD.WriteMon(gNVD.m_nCurInx,gRTC.m_mon);
   gNVD.WriteDom(gNVD.m_nCurInx,gRTC.m_dom);
   gNVD.WriteLaunchAltM(gNVD.m_nCurInx,gFD.nLaunchAltM);
   gNVD.WriteLZAltM(gNVD.m_nCurInx,gFD.nLZAltM);
   gNVD.WriteMaxAltM(gNVD.m_nCurInx,gFD.nMaxAltM);
   gNVD.WriteMaxCRCm(gNVD.m_nCurInx,0);
   gNVD.WriteMaxSRCm(gNVD.m_nCurInx,0);
   }


void ResetBkg(void)
   {
   ham_ResetBg();
   ham_SetBgMode(0);
   ham_InitText(0);
   ham_SetTextCol(1, 0);

   hel_PalBgLoad256(vario_mgd_Palette);
   ClearText(20);
   }

void DisplayDate(void)
   {
   if (gTmr.rhrs/12)
      {
      ham_DrawText(POS_DATE_X,POS_DATE_Y,"%02d:%02dPM %s %s %02d",(gTmr.rhrs-12) ?  (gTmr.rhrs-12) : 12,gTmr.rmins,szDay[gTmr.rdow-1], szMon[gRTC.m_mon-1], gRTC.m_dom);
      }
   else
      {
      ham_DrawText(POS_DATE_X,POS_DATE_Y,"%02d:%02dAM %s %s %02d",gTmr.rhrs,gTmr.rmins,szDay[gTmr.rdow-1],szMon[gRTC.m_mon-1], gRTC.m_dom);
      }
   }

void DeleteSprites(void)
   {
   gTxtAlt.DeleteSprites();
   gTxtAltAL.DeleteSprites();
   gTxtTemp.DeleteSprites();
   gTxtFTime.DeleteSprites();
   gTxtRTime.DeleteSprites();
   gTxtFilter.DeleteSprites();
   gTxtVRes.DeleteSprites();
   gTxtLArrow.DeleteSprites();
   gTxtRArrow.DeleteSprites();
   ham_ResetObj();
   }

u8 ATTR_EWRAM ATTR_ALIGNED(4) g_MapSystemBuffer[HEL_SUBSYSTEM_MAP_REQUIREDMEMORY];



void ResetScreen(void)
   {
   ClearText(20);
   hel_MapInit(g_MapSystemBuffer);
   // Load Tileset into VideoRAM
   ham_bg[1].ti = ham_InitTileSet((void*)vario_mgd_Tiles, SIZEOF_16BIT(vario_mgd_Tiles), 1, TRUE);

   // Create a map for background 0
   hel_MapCreate(1,        // Background number
                 240/8,   // width in tiles
                 1184/8,   // height in tiles
                 vario_mgd_Map,   // Pointer to source MapData
                 sizeof(u16),                  // DataTypeSize of one element from Source MapData
                 MAP_FLAGS_DEFAULT);           // Flags to control map behaviour

   // turn off boundschecking, which
   // results into map wraparound
   hel_MapSetBounds(1, 0, 0, 0, 0, 0);
   hel_MapSetPosition(1, FIXED_FROMINT(0), FIXED_FROMINT(512));

   // Turn on Background
   ham_InitBg(1, TRUE, 1, FALSE);

   gTxtAlt.Init(gFontPalette,gFont32x32Data,"0123456789+-m ",32,32,false,POS_ALT_X,POS_ALT_Y,4,5,OBJ_SIZE_32X32);
   gTxtAltAL.Init(gFontPalette,gFont16x16Data,"0123456789+-:.Cap [*]<#>lrq",16,16,false,POS_ALTAL_X,POS_ALTAL_Y,2,6,OBJ_SIZE_16X16);
   gTxtTemp.Init(gFontPalette,gFont16x16Data,"0123456789+-:.Cap [*]<#>lrq",16,16,false,POS_TEMP_X,POS_TEMP_Y,2,6,OBJ_SIZE_16X16);
   gTxtFTime.Init(gFontPalette,gFont16x16Data,"0123456789+-:.Cap [*]<#>lrq",16,16,false,POS_FTIME_X,POS_FTIME_Y,2,8,OBJ_SIZE_16X16);
   gTxtRTime.Init(gFontPalette,gFont16x16Data,"0123456789+-:.Cap [*]<#>lrq",16,16,false,POS_RTIME_X,POS_RTIME_Y,2,6,OBJ_SIZE_16X16);
   gTxtFilter.Init(gFontPalette,gFont16x16Data,"0123456789+-:.Cap [*]<#>lrq",16,16,false,POS_FILTER_X,POS_FILTER_Y,0,1,OBJ_SIZE_16X16);
   gTxtVRes.Init(gFontPalette,gFont16x16Data,"0123456789+-:.Cap [*]<#>lrq",16,16,false,POS_VRES_X,POS_VRES_Y,0,1,OBJ_SIZE_16X16);
   gTxtLArrow.Init(gFontPalette,gFont16x16Data,"0123456789+-:.Cap [*]<#>lrq",16,16,false,POS_LARROW_X,POS_LARROW_Y,0,1,OBJ_SIZE_16X16);
   gTxtRArrow.Init(gFontPalette,gFont16x16Data,"0123456789+-:.Cap [*]<#>lrq",16,16,false,POS_RARROW_X,POS_RARROW_Y,0,1,OBJ_SIZE_16X16);
   }


void VblFunc(void)
   {
   if ((gnMode == MODE_SIMULATION) || (gnMode == MODE_RUNNING))
      {
      if (gVario.m_nMode == VARIO_MODE_VOICE)
         {
         ham_SyncMixer();
         }
      ham_CopyObjToOAM();
      hel_MapTransmit();
      }
   gbVBlank = true;
   }
    
    
void StrPrintf(char* buf, char* format, ...)
    {
    va_list args;
    va_start(args,format);
    vsprintf(buf,format,args);
    va_end(args);
    }


void PrintUN(int nchars, char* buf, char* format, ...)
    {
    int cnt;
    va_list args;
    va_start(args,format);
    vsprintf(buf,format,args);
    va_end(args);
    cnt = 0;
    while (cnt < nchars-1 && buf[cnt] == '0') buf[cnt++] = ' ';
    }


void PrintSN(int nchars, char* buf, char* format, ...)
    {
    int cnt;
    char sign;
    va_list args;
    va_start(args,format);
    vsprintf(buf,format,args);
    va_end(args);
    sign = buf[0];
    cnt = 0;
    while (cnt < nchars && (buf[cnt] == '0' || buf[cnt] == sign)) buf[cnt++] = ' ';
    buf[cnt-1] = sign;
    }


void DisplaySlider(int cmPerSec)
   {
   int offset = 512;
   int delta;
   CLAMP(cmPerSec,-VARIO_MAX_CLIMB_RATE_DPY_CM,VARIO_MAX_CLIMB_RATE_DPY_CM);
      
   if (cmPerSec > 0)
      {
      delta = (cmPerSec<<9)/1600;
      offset -= delta;
      }
   else
   if (cmPerSec < 0)
      {
      delta = ((-cmPerSec)<<9)/1600;
      offset += delta;
      }

   hel_MapSetPosition(1, FIXED_FROMINT(0), FIXED_FROMINT(offset));
   }

void ClearText(int nRows)
    {
    int n;
    for (n = 0; n < nRows; n++)
        ham_DrawText(0,n,"                              ");
    }

void PrintAlt(char* szbuf, int nAlt)
    {
    CLAMP(nAlt,SENSOR_MIN_ALTITUDE_M,SENSOR_MAX_ALTITUDE_M);
    if (nAlt == 0)
       StrPrintf(szBuf,"   0m");
    else
    if (nAlt < 0)
       PrintSN(3,szbuf,"-%03dm",abs(nAlt));
    else
       PrintUN(4,szbuf,"%04dm",nAlt);
    }

void PrintAltAL(char* szbuf, int nAltAL)
    {
    CLAMP(nAltAL,-SENSOR_MAX_ALTITUDE_M,SENSOR_MAX_ALTITUDE_M);
    if (nAltAL == 0)
       StrPrintf(szbuf,"    0q");
    else
       PrintSN(4,szbuf,"%s%04dq", nAltAL > 0 ? "+" : "-", abs(nAltAL));
    }

void PrintTemperature(char* szbuf, int nTempCx10)
   {
   int dec,frac;
   CLAMP(nTempCx10,SENSOR_MIN_TEMPX10,SENSOR_MAX_TEMPX10);
   dec  = ABS(nTempCx10)/10;
   frac = ABS(nTempCx10)%10;
   if (nTempCx10 == 0)
      {
      StrPrintf(szbuf,"  0.0C");
      }
   else
   if (nTempCx10 > 0)
      {
      if (dec < 10)
         StrPrintf(szbuf,"  %d.%dC", dec, frac);
      else
         StrPrintf(szbuf," %2d.%dC", dec, frac);
      }
   else
      {
      if (dec < 10)
         StrPrintf(szbuf," -%d.%dC", dec, frac);
      else
         StrPrintf(szbuf,"-%2d.%dC", dec, frac);
      }
   }

void PrintClimbRate(char* szbuf, int crcm)
    {
    int dec,frac;
    CLAMP(crcm,-VARIO_MAX_CLIMB_RATE_LOG_CM,VARIO_MAX_CLIMB_RATE_LOG_CM);
    dec = ABS(crcm)/100;
    frac = (ABS(crcm)%100)/10;
    if (dec < 10)
        StrPrintf(szbuf,"%s %d.%dm/s", crcm >= 0? "+":"-", dec, frac);
    else
        StrPrintf(szbuf,"%s%2d.%dm/s", crcm >= 0? "+":"-", dec, frac);
    }

void PrintRealTime(char* szbuf)
   {
   CLAMP(gTmr.rhrs,0,23);
   CLAMP(gTmr.rmins,0,59);

   if (gTmr.rhrs/12)
      {
      StrPrintf(szbuf,"%02d:%02dp",(gTmr.rhrs-12) ?  (gTmr.rhrs-12) : 12,gTmr.rmins);
      }
   else
   if (gTmr.rhrs < 10)
      {
      StrPrintf(szbuf," %d:%02da",gTmr.rhrs,gTmr.rmins);
      }
   else
      {
      StrPrintf(szbuf,"%2d:%02da",gTmr.rhrs,gTmr.rmins);
      }
   }

/*
 * $Header$
 */
