#include <hel2.h>
#include "def.h"
#include "utils.h"
#include "nvd.h"
#include "i2c.h"
#include "text.h"
#include "vario.h"
#include "rtc.h"
#include "sensor.h"
#include "tmr.h"
#include "ui.h"

extern CSensor gSensor;
extern CVario  gVario;
extern CNVD    gNVD;
extern CTimer  gTimer;
extern CRTC    gRTC;
extern CI2C    gI2C;
extern FLIGHT_DATA gFD;

extern int gnMode;
extern int gnFrameCnt;

const char szMon[12][4] = {"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
const char szDay[7][4] = {"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};

char szBuf[15] = "";

CText   gTxtAlt;
CText   gTxtAltAL;
CText   gTxtTemp;
CText   gTxtFTime;
CText   gTxtRTime;
CText   gTxtFilter;
CText   gTxtVRes;
CText   gTxtLArrow;
CText   gTxtRArrow;

int gnSimClimbRate;
int gnLogRec;
int gnParSel;
int gbVBlank;

#include "gfx/vario_mgd.map.c"
#include "gfx/vario_mgd.raw.c"
#include "gfx/vario_mgd.pal.c"

#include "gfx/font.c"


void SimEventHandler(void) {
	int nFrmSlot;
	static int keyDelta;
	static int iCumAlt;
	static int iSimAlt = gFD.nLaunchAltM;

	//gSensor.ReadSampleContinuous();
	if (gFD.nMaxCRCm < gnSimClimbRate) gFD.nMaxCRCm = gnSimClimbRate;
	if (gFD.nMaxSRCm < gnSimClimbRate) gFD.nMaxSRCm = gnSimClimbRate;

	if ((gTmr.runFlag == false) && (ABS(iSimAlt - gFD.nLaunchAltM) > NVD_ATO_THRESHOLD)) {
		gTmr.runFlag = true;
		gNVD.WriteLHr(gNVD.m_nCurInx,gTmr.rhrs);
		gNVD.WriteLMin(gNVD.m_nCurInx,gTmr.rmins);
	}

	if (gTmr.varioTickFlag == true) {
		gTmr.varioTickFlag = false;
		gVario.Chirp(gnSimClimbRate);
	}

	if (gTmr.secFlag == true) {
		gTmr.secFlag = false;

		iCumAlt += gnSimClimbRate;
		iSimAlt += iCumAlt/100;
		if (ABS(iCumAlt) > 100) iCumAlt = 0;
	}

	if (gbVBlank == true) {
		gbVBlank = false;
		gnFrameCnt++;
		ham_UpdatePad();
		DisplaySlider(gnSimClimbRate);

		if (Pad.Pressed.Start) {
			VARIO_SOUND_RESET();
			DeleteSprites();
			ResetBkg();
			gSensor.m_nCmPerSec = 0;
			gVario.m_nCmPerSec = 0;
			InitLogMode();
		}

		if (Pad.Held.Down && gnSimClimbRate >= (-VARIO_MAX_CLIMB_RATE_DPY_CM+keyDelta))	{
			if (gnFrameCnt % 3 == 0) {
				gnSimClimbRate -= keyDelta++;
				CLAMP(keyDelta,0,5);
			}
		} else if (Pad.Held.Up && gnSimClimbRate <= (VARIO_MAX_CLIMB_RATE_DPY_CM-keyDelta)) {
			if (gnFrameCnt % 3 == 0) {
				gnSimClimbRate += keyDelta++;
				CLAMP(keyDelta,0,5);
			}
		} else {
			keyDelta = 1;
		}

		nFrmSlot = gnFrameCnt % 64;
		switch (nFrmSlot) {
			case 0 :
				//PrintAlt(szBuf,gSensor.m_nAltM);
				PrintAlt(szBuf,iSimAlt);
				gTxtAlt.SetText(szBuf);
				gTxtAlt.DisplayStatic();
				break;

			case 2 :
				PrintAltAL(szBuf, iSimAlt - gFD.nLaunchAltM);
				gTxtAltAL.SetText(szBuf);
				gTxtAltAL.DisplayStatic();
				break;

			case 4 :
				PrintTemperature(szBuf,gSensor.m_nTempCx10);
				gTxtTemp.SetText(szBuf);
				gTxtTemp.DisplayStatic();
				break;

			case 6 :
				PrintRealTime(szBuf);
				gTxtRTime.SetText(szBuf);
				gTxtRTime.DisplayStatic();
				break;

			case 8 :
				if (gTmr.runFlag == true) {
					StrPrintf(szBuf,"%02d:%02d:%02d", gTmr.fhrs, gTmr.fmins, gTmr.fsecs);
					gTxtFTime.SetText(szBuf);
					gTxtFTime.DisplayStatic();
				}

			default :
				break;
		}
	}
}



void RunEventHandler(void)
   {
   int nFrmSlot;
   u32 sTick;

   if ((gTmr.runFlag == false) && (ABS(gSensor.m_nAltM - gFD.nLaunchAltM) > NVD_ATO_THRESHOLD))
      {
      gTmr.runFlag = true;
      gNVD.WriteLHr(gNVD.m_nCurInx,gTmr.rhrs);
      gNVD.WriteLMin(gNVD.m_nCurInx,gTmr.rmins);
      }

   sTick = gTmr.ticks;
   gSensor.ReadSampleContinuous();

   if (gTmr.varioTickFlag == true)
      {
      gTmr.varioTickFlag = false;
      gVario.Chirp(gSensor.m_nCmPerSec);
      }

   if (gTmr.runFlag == true) 
      {
      if (gFD.nMaxAltM < gSensor.m_nAltM)
         gFD.nMaxAltM = gSensor.m_nAltM;
      if (gFD.nMaxCRCm < gSensor.m_nCmPerSec)
         gFD.nMaxCRCm = gSensor.m_nCmPerSec;
      if (gFD.nMaxSRCm < -gSensor.m_nCmPerSec)
         gFD.nMaxSRCm = -gSensor.m_nCmPerSec;

      if (gTmr.secFlag == true)
         {
         gTmr.secFlag = false;
         StrPrintf(szBuf,"%02d:%02d:%02d",gTmr.fhrs,gTmr.fmins,gTmr.fsecs);
         gTxtFTime.SetText(szBuf);
         gTxtFTime.DisplayStatic();
         gNVD.WriteFHrs(gNVD.m_nCurInx,gTmr.fhrs);
         gNVD.WriteFMin(gNVD.m_nCurInx,gTmr.fmins);
         gNVD.WriteFSec(gNVD.m_nCurInx,gTmr.fsecs);
         }
      }

   if (gbVBlank == true)
      {
      gbVBlank = false;
      gnFrameCnt++;
      ham_UpdatePad();

      DisplaySlider(gSensor.m_nCmPerSec);
      if (Pad.Pressed.Select)
         {
         ResetLogRecord();
         gNVD.WriteLHr(gNVD.m_nCurInx,gTmr.rhrs);
         gNVD.WriteLMin(gNVD.m_nCurInx,gTmr.rmins);
         }
      else
      if (Pad.Pressed.B)
         {
         if (gSensor.m_nKLPFC == SENSOR_KLPFC_L)
            {
            gSensor.m_nKLPFC = SENSOR_KLPFC_M;
            gTxtFilter.SetText("*");
            }
         else
         if (gSensor.m_nKLPFC == SENSOR_KLPFC_M)
            {
            gSensor.m_nKLPFC = SENSOR_KLPFC_H;
            gTxtFilter.SetText("]");
            }
         else
            {
            gSensor.m_nKLPFC = SENSOR_KLPFC_L;
            gTxtFilter.SetText("[");
            }
         gNVD.WriteKLPFC(gSensor.m_nKLPFC);
         gTxtFilter.DisplayStatic();
         }
      else
      if (Pad.Pressed.A)
         {
         if (gVario.m_nResolutionCm == VARIO_RESOLUTION_L)
            {
            gVario.m_nResolutionCm = VARIO_RESOLUTION_M;
            gTxtVRes.SetText("#");
            }
         else
         if (gVario.m_nResolutionCm == VARIO_RESOLUTION_M)
            {
            gVario.m_nResolutionCm = VARIO_RESOLUTION_H;
            gTxtVRes.SetText(">");
            }
         else
            {
            gVario.m_nResolutionCm = VARIO_RESOLUTION_L;
            gTxtVRes.SetText("<");
            }
         gNVD.WriteVarioRes(gVario.m_nResolutionCm);
         gTxtVRes.DisplayStatic();
         }
      else
      if (Pad.Pressed.Start) // GO TO LOG MODE
         {
         VARIO_SOUND_RESET();
         DeleteSprites();
         ResetBkg();
         if (gTmr.fhrs == 0 && gTmr.fmins == 0 && gTmr.fsecs == 0)
            {
            gNVD.DeleteFlightRecord(gNVD.m_nCurInx);
            }
         gSensor.m_nCmPerSec = 0;
         gVario.m_nCmPerSec = 0;
         InitLogMode();
         }

      nFrmSlot = gnFrameCnt % 64;
      switch (nFrmSlot)
         {
         case 0 :
         PrintAlt(szBuf,gSensor.m_nAltM);
         gTxtAlt.SetText(szBuf);
         gTxtAlt.DisplayStatic();
         break;


         case 2 :
         PrintAltAL(szBuf,gSensor.m_nAltM - gFD.nLaunchAltM);
         gTxtAltAL.SetText(szBuf);
         gTxtAltAL.DisplayStatic();
         break;

         case 4 :
         PrintTemperature(szBuf,gSensor.m_nTempCx10);
         gTxtTemp.SetText(szBuf);
         gTxtTemp.DisplayStatic();
         break;

         case 6 :
         PrintRealTime(szBuf);
         gTxtRTime.SetText(szBuf);
         gTxtRTime.DisplayStatic();
         // ham_DrawText(0,3,"RTM%03d",(gTmr.ticks - stick)*5);
         break;

         case 8 :
         if (gTmr.runFlag == true)
            {
            gNVD.WriteMaxAltM(gNVD.m_nCurInx,gFD.nMaxAltM);
            gNVD.WriteLZAltM(gNVD.m_nCurInx,gSensor.m_nAltM);
            // ham_DrawText(0,4,"NVD%03d",(gTmr.ticks - stick)*5);
            }
         break;

         case 10 :
         if (gTmr.runFlag == true)
            {
            gNVD.WriteMaxCRCm(gNVD.m_nCurInx,gFD.nMaxCRCm);
            gNVD.WriteMaxSRCm(gNVD.m_nCurInx,gFD.nMaxSRCm);
            }
         break;

         default :
         break;
         }
      }
   }



void LogEventHandler(void)
   {
   int ix;
   
   if (gbVBlank == true)
      {
      gbVBlank = false;
      gnFrameCnt++;
      if ((gnFrameCnt & 63) == 0)
         DisplayDate();

      ham_UpdatePad();

      if (Pad.Pressed.Left)
         {
         gnLogRec =  (gnLogRec > 0)? gnLogRec-1 : NVD_MAX_FLIGHTS-1;
         DisplayLogRecord(gnLogRec);
         }
      else
      if (Pad.Pressed.Right)
         {
         gnLogRec = (gnLogRec < NVD_MAX_FLIGHTS-1)?  gnLogRec+1 : 0;
         DisplayLogRecord(gnLogRec);
         }
      else
      if (Pad.Pressed.Select)
         {
         ix = gNVD.m_nCurInx - gnLogRec;
         if (ix < 0)
            ix += NVD_MAX_FLIGHTS;
         gNVD.DeleteFlightRecord(ix);
         DisplayLogRecord(gnLogRec);
         }
      else
      if (Pad.Pressed.Start)
         {
         InitSetMode();
         }
      }
  }


void SetEventHandler(void)
   {
   if (gbVBlank == true)
      {
      gbVBlank = false;
      gnFrameCnt++;
      ham_UpdatePad();
      if (Pad.Pressed.Up)
         {
         gnParSel = gnParSel ? gnParSel-1 : SEL_RTC_MIN;
         DisplaySettings();
         }
      else
      if (Pad.Pressed.Down)
         {
         gnParSel = gnParSel < SEL_RTC_MIN ? gnParSel+1 : SEL_ALTITUDE;
         DisplaySettings();
         }
      else
      if (Pad.Pressed.Left)
         {
         switch (gnParSel)
            {
            case SEL_ALTITUDE :
            default :
            if (gSensor.m_nPZTblSel > SENSOR_MIN_PZTBL_SEL)
               {
               gSensor.m_nPZTblSel--;
               gNVD.WritePZTblSel(gSensor.m_nPZTblSel);
               gSensor.GetAveragedAltitude(4);
               }
            break;

            case SEL_WINDOW_SECS :
            if (gSensor.m_nWindowSecs > SENSOR_MIN_WINDOW_SECS)
               {
               gSensor.m_nWindowSecs--;
               gSensor.m_nWindowSamples = gSensor.m_nWindowSecs * SENSOR_SAMPLES_PER_SEC;
               gNVD.WriteWindowSecs(gSensor.m_nWindowSecs);
               }
            break;

            case SEL_SINK_THRESH :
            if (gVario.m_nSinkThresholdCm >= (VARIO_MIN_SINKTONE_THRESHOLD_CM+50))
               {
               gVario.m_nSinkThresholdCm -= 50;
               gNVD.WriteSinkThresholdCm(gVario.m_nSinkThresholdCm);
               }
            break;

            case SEL_RTC_YAR :
               gRTC.DecrYar();
            break;

            case SEL_RTC_MON :
               gRTC.DecrMon();
            break;

            case SEL_RTC_DOM :
               gRTC.DecrDom();
            break;

            case SEL_RTC_DOW :
               gRTC.DecrDow();
               gTimer.SetRealTime(gRTC.m_dow,gRTC.m_hrs,gRTC.m_min,0);
            break;

            case SEL_RTC_HRS :
               gRTC.DecrHrs();
               gTimer.SetRealTime(gRTC.m_dow,gRTC.m_hrs,gRTC.m_min,0);
            break;

            case SEL_RTC_MIN :
               gRTC.DecrMin();
               gTimer.SetRealTime(gRTC.m_dow,gRTC.m_hrs,gRTC.m_min,0);
            break;
            }
         DisplaySettings();
         }
      else
      if (Pad.Pressed.Right)
         {
         switch (gnParSel)
            {
            case SEL_ALTITUDE :
            default :
            if (gSensor.m_nPZTblSel < SENSOR_MAX_PZTBL_SEL)
               {
               gSensor.m_nPZTblSel++;
               gNVD.WritePZTblSel(gSensor.m_nPZTblSel);
               gSensor.GetAveragedAltitude(4);
               }
            break;

            case SEL_WINDOW_SECS :
            if (gSensor.m_nWindowSecs < SENSOR_MAX_WINDOW_SECS)
               {
               gSensor.m_nWindowSecs++;
               gSensor.m_nWindowSamples = gSensor.m_nWindowSecs*SENSOR_SAMPLES_PER_SEC;
               gNVD.WriteWindowSecs(gSensor.m_nWindowSecs);
               }
            break;

            case SEL_SINK_THRESH :
            if (gVario.m_nSinkThresholdCm <= (VARIO_MAX_SINKTONE_THRESHOLD_CM-50))
               {
               gVario.m_nSinkThresholdCm += 50;
               gNVD.WriteSinkThresholdCm(gVario.m_nSinkThresholdCm);
               }
            break;

            case SEL_RTC_YAR :
               gRTC.IncrYar();
            break;

            case SEL_RTC_MON :
               gRTC.IncrMon();
            break;

            case SEL_RTC_DOM :
               gRTC.IncrDom();
            break;

            case SEL_RTC_DOW :
               gRTC.IncrDow();
               gTimer.SetRealTime(gRTC.m_dow,gRTC.m_hrs,gRTC.m_min,0);
            break;

            case SEL_RTC_HRS :
               gRTC.IncrHrs();
               gTimer.SetRealTime(gRTC.m_dow,gRTC.m_hrs,gRTC.m_min,0);
            break;

            case SEL_RTC_MIN :
               gRTC.IncrMin();
               gTimer.SetRealTime(gRTC.m_dow,gRTC.m_hrs,gRTC.m_min,0);
            break;
            }
         DisplaySettings();
         }
      else
      if (Pad.Pressed.A)
         {
         gSensor.m_nPZTblSel = SENSOR_DFL_PZTBL_SEL;
         gSensor.m_nTempCorrInx = SENSOR_DFL_TEMP_CORR_INX;
         gNVD.WritePZTblSel(gSensor.m_nPZTblSel);
         gSensor.GetAveragedAltitude(4);
         DisplaySettings();
         }
      else
      if (Pad.Pressed.Start)
         {
         InitRunMode();
         }
      else
      if (Pad.Pressed.L)
         {
         InitSimMode();
         }

      if ((gnFrameCnt & 63) == 0)
         {
         DisplayDate();
         }
      }
   }

   
void InitLogMode()
   {
   gnMode = MODE_LOG;
   gnLogRec = 0;
   DisplayLogRecord(gnLogRec);
   }

void InitSetMode()
   {
   gnMode = MODE_SETTINGS;
   gnParSel = SEL_ALTITUDE;
   gSensor.CalibrateT0();
   ClearText(20);
   DisplaySettings();
   }

void InitSimMode()
   {
   gnMode = MODE_SIMULATION;
   gSensor.InitData();
#ifdef SENSOR_SCP1000
   gSensor.SetSampleMode(I2C_PORT1,SENSOR_SAMPLE_CONTINUOUS);
   BusyWait(60);
   gSensor.SetSampleMode(I2C_PORT2,SENSOR_SAMPLE_CONTINUOUS);
#endif
   ResetScreen();
   ClearText(20);

   gTxtLArrow.SetText("l");
   gTxtLArrow.DisplayStatic();
   gTxtRArrow.SetText("r");
   gTxtRArrow.DisplayStatic();

   PrintAlt(szBuf, 300); //gSensor.m_nAltM);
   //PrintAlt(szBuf, gSensor.m_nAltM);

   gTxtAlt.SetText(szBuf);
   gTxtAlt.DisplayStatic();
   PrintTemperature(szBuf,gSensor.m_nTempCx10);
   gTxtTemp.SetText(szBuf);
   gTxtTemp.DisplayStatic();

   PrintAltAL(szBuf,0);
   gTxtAltAL.SetText(szBuf);
   gTxtAltAL.DisplayStatic();

   gTxtAltAL.SetText("------");
   gTxtAltAL.DisplayStatic();

   gTxtFTime.SetText("--:--:--");
   gTxtFTime.DisplayStatic();
   gTxtVRes.SetText("-");
   gTxtVRes.DisplayStatic();
   gTxtFilter.SetText("-");
   gTxtFilter.DisplayStatic();

   gnSimClimbRate = 0;
   gVario.m_nCmPerSec = 0;
   gFD.nMaxAltM = gFD.nLZAltM = gFD.nLaunchAltM = 300;
   }


void InitRunMode()
   {
   gnMode = MODE_RUNNING;
   gVario.m_nCmPerSec = 0;
   gSensor.InitData();
#ifdef SENSOR_SCP1000
   gSensor.SetSampleMode(I2C_PORT1,SENSOR_SAMPLE_CONTINUOUS);
   BusyWait(60);
   gSensor.SetSampleMode(I2C_PORT2,SENSOR_SAMPLE_CONTINUOUS);
#endif
   ResetScreen();
   ClearText(20);
   gTxtLArrow.SetText("l");
   gTxtLArrow.DisplayStatic();
   gTxtRArrow.SetText("r");
   gTxtRArrow.DisplayStatic();
   PrintAlt(szBuf,gSensor.m_nAltM);
   gTxtAlt.SetText(szBuf);
   gTxtAlt.DisplayStatic();
   PrintTemperature(szBuf,gSensor.m_nTempCx10);
   gTxtTemp.SetText(szBuf);
   gTxtTemp.DisplayStatic();
   PrintAltAL(szBuf,0);
   gTxtAltAL.SetText(szBuf);
   gTxtAltAL.DisplayStatic();
   PrintRealTime(szBuf);
   gTxtRTime.SetText(szBuf);
   gTxtRTime.DisplayStatic();

   if (gVario.m_nResolutionCm == VARIO_RESOLUTION_H)
      gTxtVRes.SetText(">");
   else
   if (gVario.m_nResolutionCm == VARIO_RESOLUTION_M)
      gTxtVRes.SetText("#");
   else
      gTxtVRes.SetText("<");
   gTxtVRes.DisplayStatic();
   
   if (gSensor.m_nKLPFC == SENSOR_KLPFC_H)
      gTxtFilter.SetText("]");
   else
   if (gSensor.m_nKLPFC == SENSOR_KLPFC_M)
      gTxtFilter.SetText("*");
   else
      gTxtFilter.SetText("[");
   gTxtFilter.DisplayStatic();

   gNVD.m_nCurInx++;
   if (gNVD.m_nCurInx == NVD_MAX_FLIGHTS)
      gNVD.m_nCurInx = 0;
   gNVD.WriteCurInx(gNVD.m_nCurInx);
   ResetLogRecord();
   gTimer.Reset();
   }

void DisplaySettings()
    {
    int n;

    ham_DrawText(10, 1,"SETTINGS");
    for (n = 0; n <= SEL_RTC_MIN; n++)
        ham_DrawText(1,SEL_Y_ORIG+n,"%s",gnParSel == n ? "x " : "  ");

    n = 0;
    ham_DrawText(3, SEL_Y_ORIG+n, "ALTITUDE          : %04dm",gSensor.m_nAltM);n++;
    ham_DrawText(3, SEL_Y_ORIG+n, "AVERAGING WINDOW  : %ds",gSensor.m_nWindowSecs);n++;
    ham_DrawText(3, SEL_Y_ORIG+n, "SINKTONE THRESHOLD: %d.%dm/s",gVario.m_nSinkThresholdCm/100,(gVario.m_nSinkThresholdCm%100)/10);n++;
    ham_DrawText(3, SEL_Y_ORIG+n, "YEAR              : %d",2000+gRTC.m_yar);n++;
    ham_DrawText(3, SEL_Y_ORIG+n, "MONTH             : %s",szMon[gRTC.m_mon-1]);n++;
    ham_DrawText(3, SEL_Y_ORIG+n, "DAY OF MONTH      : %02d",gRTC.m_dom);n++;
    ham_DrawText(3, SEL_Y_ORIG+n, "DAY OF WEEK       : %s",szDay[gRTC.m_dow-1]);n++;
    ham_DrawText(3, SEL_Y_ORIG+n, "HOUR              : %02d",gRTC.m_hrs);n++;
    ham_DrawText(3, SEL_Y_ORIG+n, "MINUTE            : %02d",gRTC.m_min);n++;
    }


void DisplayLogRecord(int nrec)
{
	int ix, row; //lhr

	ClearText(20);
	DisplayDate();
	ham_DrawText(10,1,"Flight %d", nrec+1);
	ix = gNVD.m_nCurInx - nrec;
	if (ix < 0)
		ix += NVD_MAX_FLIGHTS;

	/*
	lhr = gNVD.ReadLHr(ix);
	if (lhr/12)
	{
	ham_DrawText(LOG_X,LOG_Y+row,"Launch        %02d:%02dPM",(lhr-12) ? (lhr-12) : 12,gNVD.ReadLMin(ix));
	}
	else
	{
	ham_DrawText(LOG_X,LOG_Y+row,"Launch        %02d:%02dAM",lhr,gNVD.ReadLMin(ix));
	}
	*/
	row = 0;
	ham_DrawText(LOG_X, LOG_Y+row, "Date        : 20%02d %s %02d", gNVD.ReadYear(ix), szMon[gNVD.ReadMon(ix)-1], gNVD.ReadDom(ix));  row++;
	ham_DrawText(LOG_X, LOG_Y+row, "Launch      : %02d:%02d", gNVD.ReadLHr(ix), gNVD.ReadLMin(ix)); row++;
	ham_DrawText(LOG_X, LOG_Y+row, "Duration    : %02d:%02d:%02d",gNVD.ReadFHrs(ix),gNVD.ReadFMin(ix), gNVD.ReadFSec(ix));	row++;
	PrintAlt(szBuf, gNVD.ReadLaunchAltM(ix));
	ham_DrawText(LOG_X, LOG_Y+row, "Launch Alt  : %s", szBuf); row++;
	PrintAlt(szBuf,gNVD.ReadMaxAltM(ix));
	ham_DrawText(LOG_X, LOG_Y+row, "Maximum Alt : %s", szBuf); row++;
	PrintAlt(szBuf,gNVD.ReadLZAltM(ix));
	ham_DrawText(LOG_X, LOG_Y+row, "LZ Alt      : %s", szBuf); row++;
	PrintClimbRate(szBuf,gNVD.ReadMaxCRCm(ix));
	ham_DrawText(LOG_X, LOG_Y+row, "Max Climb   : %s", szBuf); row++;
	PrintClimbRate(szBuf,gNVD.ReadMaxSRCm(ix));
	ham_DrawText(LOG_X, LOG_Y+row, "Max Sink    : %s", szBuf); row++;
}


void ResetLogRecord()
   {
   gFD.nMaxCRCm = 0;
   gFD.nMaxSRCm  = 0;

   gFD.nMaxAltM = gFD.nLZAltM = gFD.nLaunchAltM = gSensor.m_nAltM;
   gNVD.WriteYear(gNVD.m_nCurInx,gRTC.m_yar);
   gNVD.WriteMon(gNVD.m_nCurInx,gRTC.m_mon);
   gNVD.WriteDom(gNVD.m_nCurInx,gRTC.m_dom);
   gNVD.WriteLaunchAltM(gNVD.m_nCurInx,gFD.nLaunchAltM);
   gNVD.WriteLZAltM(gNVD.m_nCurInx,gFD.nLZAltM);
   gNVD.WriteMaxAltM(gNVD.m_nCurInx,gFD.nMaxAltM);
   gNVD.WriteMaxCRCm(gNVD.m_nCurInx,0);
   gNVD.WriteMaxSRCm(gNVD.m_nCurInx,0);
   }

void ResetBkg(void)
   {
   ham_ResetBg();
   ham_SetBgMode(0);
   ham_InitText(0);
   ham_SetTextCol(1, 0);
   hel_PalBgLoad256(vario_mgd_Palette);
   ClearText(20);
   }

void DisplayDate(void)
   {
   if (gTmr.rhrs/12)
      {
      ham_DrawText(POS_DATE_X,POS_DATE_Y,"%02d:%02dPM %s %s %02d",(gTmr.rhrs-12) ?  (gTmr.rhrs-12) : 12,gTmr.rmins,szDay[gTmr.rdow-1], szMon[gRTC.m_mon-1], gRTC.m_dom);
      }
   else
      {
      ham_DrawText(POS_DATE_X,POS_DATE_Y,"%02d:%02dAM %s %s %02d",gTmr.rhrs,gTmr.rmins,szDay[gTmr.rdow-1],szMon[gRTC.m_mon-1], gRTC.m_dom);
      }
   }

void DeleteSprites(void)
   {
   gTxtAlt.DeleteSprites();
   gTxtAltAL.DeleteSprites();
   gTxtTemp.DeleteSprites();
   gTxtFTime.DeleteSprites();
   gTxtRTime.DeleteSprites();
   gTxtFilter.DeleteSprites();
   gTxtVRes.DeleteSprites();
   gTxtLArrow.DeleteSprites();
   gTxtRArrow.DeleteSprites();
   ham_ResetObj();
   }

u8 ATTR_EWRAM ATTR_ALIGNED(4) g_MapSystemBuffer[HEL_SUBSYSTEM_MAP_REQUIREDMEMORY];



void ResetScreen(void)
   {
   ClearText(20);
   hel_MapInit(g_MapSystemBuffer);
   // Load Tileset into VideoRAM
   ham_bg[1].ti = ham_InitTileSet((void*)vario_mgd_Tiles, SIZEOF_16BIT(vario_mgd_Tiles), 1, TRUE);

   // Create a map for background 0
   hel_MapCreate(1,        // Background number
                 240/8,   // width in tiles
                 1184/8,   // height in tiles
                 vario_mgd_Map,   // Pointer to source MapData
                 sizeof(u16),                  // DataTypeSize of one element from Source MapData
                 MAP_FLAGS_DEFAULT);           // Flags to control map behaviour

   // turn off boundschecking, which
   // results into map wraparound
   hel_MapSetBounds(1, 0, 0, 0, 0, 0);
   hel_MapSetPosition(1, FIXED_FROMINT(0), FIXED_FROMINT(512));

   // Turn on Background
   ham_InitBg(1, TRUE, 1, FALSE);

   gTxtAlt.Init(gFontPalette,gFont32x32Data,"0123456789+-m ",32,32,false,POS_ALT_X,POS_ALT_Y,4,5,OBJ_SIZE_32X32);
   gTxtAltAL.Init(gFontPalette,gFont16x16Data,"0123456789+-:.Cap [*]<#>lrq",16,16,false,POS_ALTAL_X,POS_ALTAL_Y,2,6,OBJ_SIZE_16X16);
   gTxtTemp.Init(gFontPalette,gFont16x16Data,"0123456789+-:.Cap [*]<#>lrq",16,16,false,POS_TEMP_X,POS_TEMP_Y,2,6,OBJ_SIZE_16X16);
   gTxtFTime.Init(gFontPalette,gFont16x16Data,"0123456789+-:.Cap [*]<#>lrq",16,16,false,POS_FTIME_X,POS_FTIME_Y,2,8,OBJ_SIZE_16X16);
   gTxtRTime.Init(gFontPalette,gFont16x16Data,"0123456789+-:.Cap [*]<#>lrq",16,16,false,POS_RTIME_X,POS_RTIME_Y,2,6,OBJ_SIZE_16X16);
   gTxtFilter.Init(gFontPalette,gFont16x16Data,"0123456789+-:.Cap [*]<#>lrq",16,16,false,POS_FILTER_X,POS_FILTER_Y,0,1,OBJ_SIZE_16X16);
   gTxtVRes.Init(gFontPalette,gFont16x16Data,"0123456789+-:.Cap [*]<#>lrq",16,16,false,POS_VRES_X,POS_VRES_Y,0,1,OBJ_SIZE_16X16);
   gTxtLArrow.Init(gFontPalette,gFont16x16Data,"0123456789+-:.Cap [*]<#>lrq",16,16,false,POS_LARROW_X,POS_LARROW_Y,0,1,OBJ_SIZE_16X16);
   gTxtRArrow.Init(gFontPalette,gFont16x16Data,"0123456789+-:.Cap [*]<#>lrq",16,16,false,POS_RARROW_X,POS_RARROW_Y,0,1,OBJ_SIZE_16X16);
   }


void VblFunc(void)
    {
    gbVBlank = true;
    if ((gnMode == MODE_SIMULATION) || (gnMode == MODE_RUNNING))
        {
        ham_CopyObjToOAM();
        }
    hel_MapTransmit();
    }
    
    
void StrPrintf(char* buf, char* format, ...)
    {
    va_list args;
    va_start(args,format);
    vsprintf(buf,format,args);
    va_end(args);
    }

void PrintUN(int nchars, char* buf, char* format, ...)
    {
    int cnt;
    va_list args;
    va_start(args,format);
    vsprintf(buf,format,args);
    va_end(args);
    cnt = 0;
    while (cnt < nchars-1 && buf[cnt] == '0') buf[cnt++] = ' ';
    }


void PrintSN(int nchars, char* buf, char* format, ...)
    {
    int cnt;
    char sign;
    va_list args;
    va_start(args,format);
    vsprintf(buf,format,args);
    va_end(args);
    sign = buf[0];
    cnt = 0;
    while (cnt < nchars && (buf[cnt] == '0' || buf[cnt] == sign)) buf[cnt++] = ' ';
    buf[cnt-1] = sign;
    }


void DisplaySlider(int cmPerSec)
   {
   int offset = 512;
   int delta;
   CLAMP(cmPerSec,-VARIO_MAX_CLIMB_RATE_DPY_CM,VARIO_MAX_CLIMB_RATE_DPY_CM);
 
   if (cmPerSec > 0)
      {
	  delta = (cmPerSec<<9)/1600;
      offset -= delta;
      }
   else
   if (cmPerSec < 0)
      {
      delta = ((-cmPerSec)<<9)/1600;
      offset += delta;
      }

   hel_MapSetPosition(1, FIXED_FROMINT(0), FIXED_FROMINT(offset));
   }

void ClearText(int nRows)
    {
    int n;
    for (n = 0; n < nRows; n++)
        ham_DrawText(0,n,"                              ");
    }

void PrintAlt(char* szbuf, int nAlt)
    {
    CLAMP(nAlt,SENSOR_MIN_ALTITUDE_M,SENSOR_MAX_ALTITUDE_M);
    PrintUN(4,szbuf,"%04dm",nAlt);
    }

void PrintAltAL(char* szbuf, int nAltAL)
    {
    CLAMP(nAltAL,-SENSOR_MAX_ALTITUDE_M,SENSOR_MAX_ALTITUDE_M);
    if (nAltAL == 0)
       StrPrintf(szbuf,"    0q");
    else
       PrintSN(4,szbuf,"%s%04dq", nAltAL > 0 ? "+" : "-", abs(nAltAL));
    }

void PrintTemperature(char* szbuf, int nTempCx10)
   {
   int dec,frac;
   CLAMP(nTempCx10,SENSOR_MIN_TEMPX10,SENSOR_MAX_TEMPX10);
   dec  = ABS(nTempCx10)/10;
   frac = ABS(nTempCx10)%10;
   if (nTempCx10 == 0)
      {
      StrPrintf(szbuf,"  0.0C");
      }
   else
   if (nTempCx10 > 0)
      {
      if (dec < 10)
         StrPrintf(szbuf,"  %d.%dC", dec, frac);
      else
         StrPrintf(szbuf," %2d.%dC", dec, frac);
      }
   else
      {
      if (dec < 10)
         StrPrintf(szbuf," -%d.%dC", dec, frac);
      else
         StrPrintf(szbuf,"-%2d.%dC", dec, frac);
      }
   }

void PrintClimbRate(char* szbuf, int crcm)
    {
    int dec,frac;
    CLAMP(crcm,-VARIO_MAX_CLIMB_RATE_LOG_CM,VARIO_MAX_CLIMB_RATE_LOG_CM);
    dec = ABS(crcm)/100;
    frac = (ABS(crcm)%100)/10;
    if (dec < 10)
        StrPrintf(szbuf,"%s %d.%dm/s", crcm >= 0? "+":"-", dec, frac);
    else
        StrPrintf(szbuf,"%s%2d.%dm/s", crcm >= 0? "+":"-", dec, frac);
    }

void PrintRealTime(char* szbuf) {
	if (gTmr.rhrs/12) {
		StrPrintf(szbuf, "%02d:%02dp", (gTmr.rhrs-12) ?  (gTmr.rhrs-12) : 12, gTmr.rmins);
	} else
		if (gTmr.rhrs < 10)	{
			StrPrintf(szbuf," %d:%02da", gTmr.rhrs, gTmr.rmins);
		} else {
			StrPrintf(szbuf,"%2d:%02da",gTmr.rhrs, gTmr.rmins);
		}
}

/*
 * $Header$
 */
