#include "vario.h"
#include "voice.h"
#include "tmr.h"

extern CVoice gVoice;
extern TIMER  gTmr;


void CVario::Init()
   {
   SetMode(VARIO_MODE_CHIRP);
   m_nSinkThresholdCm = VARIO_DFL_SINKTONE_THRESHOLD_CM;
   m_nResolutionCm = VARIO_DFL_RESOLUTION;
   }

void CVario::SetMode(int nMode)
   {
   m_nMode = nMode;
   if (m_nMode == VARIO_MODE_CHIRP)
      {
      M_SNDCNT2_DSOUNDA_OUTPUT_R_ENABLE(0);
      M_SNDCNT2_DSOUNDA_OUTPUT_L_ENABLE(0);

      if (gVoice.m_wave[gVoice.m_nVoiceSel]->playing)
         {
         ham_KillSample(gVoice.m_wave[gVoice.m_nVoiceSel]);
         ham_UpdateMixer();
         }
      ham_DeInitMixer();
      M_SNDCNT3_SOUND_MASTER_ON;
      M_SNDCNT1_INIT_ALL_ON;
      M_SNDCNT2_SOUND1TO4_OUTPUTRATIO_SET(2);	

      m_nNumSweeps   = 0; // no sweep
      m_nSweepTime   = 0; // 0 : disable sweep
      m_nSweepDir    = 1; // set if sweep is disabled //

      m_nSoundLen    = 16; // 0.1875s beep
      m_nInitVolume  = 15;
      m_nDutyCycle   = 2;
      m_nVolumeSteps = 0; // no Envelope
      m_nFadeMode    = 0;    // don't care
      m_nLoopMode    = VARIO_LOOP_TIMED;

      m_nFreqReg = 0;

      m_nTickCnt = 1;
      m_nTotalTicks = 0;
      M_SND1CNT1_SETUP(m_nSweepTime,m_nNumSweeps,m_nSweepDir);
      M_SND1CNT2_SETUP(m_nInitVolume,m_nFadeMode,m_nVolumeSteps,m_nSoundLen,m_nDutyCycle);
      }
   else
      {
      VARIO_SOUND_RESET();
      gVoice.Init();
      }
   }

void CVario::Voice(int nCmPerSec)
   {
   int nDelta;

   CLAMP(nCmPerSec,-VARIO_MAX_CLIMB_RATE_DPY_CM,VARIO_MAX_CLIMB_RATE_DPY_CM);
   // abnormal sink - warning messages
   if (nCmPerSec < -m_nSinkThresholdCm)
      {
      m_nCmPerSec = nCmPerSec;
      if (nCmPerSec <= -VARIO_MAX_SINKTONE_THRESHOLD_CM)
         {
         gVoice.m_nVoiceSel = S_ATTENTION;
         }
      else
         {
         //if ((gTmr.tickCntr & 0xffff) > 0 && (gTmr.tickCntr & 0xffff) < 100)
         //   gVoice.m_nVoiceSel = S_SINKING;
         //else
         //   gVoice.m_nVoiceSel = S_WRUSINKING;
         gVoice.m_nVoiceSel = S_SINKING;
         }
      }
   else
   // normal sink, stay quiet
   if (nCmPerSec <  VARIO_CORE_CLIMB_RATE_CM)
      {
      m_nCmPerSec = nCmPerSec;
      gVoice.m_nVoiceSel = -1;
      }
   else // climbing, call out the climb rate
      {   
      m_nCmPerSec = nCmPerSec;
      if (m_nCmPerSec >= 1600)
         {
         gVoice.m_nVoiceSel = S_16;
         nDelta = 50;
         }
      else
      if (m_nCmPerSec >= 1500)
         {
         gVoice.m_nVoiceSel = S_15;
         nDelta = m_nCmPerSec - 1500;
         }
      else
      if (m_nCmPerSec >= 1400)
         {
         gVoice.m_nVoiceSel = S_14;
         nDelta = m_nCmPerSec - 1400;
         }
      else
      if (m_nCmPerSec >= 1300)
         {
         gVoice.m_nVoiceSel = S_13;
         nDelta = m_nCmPerSec - 1300;
         }
      else
      if (m_nCmPerSec >= 1200)
         {
         gVoice.m_nVoiceSel = S_12;
         nDelta = m_nCmPerSec - 1200;
         }
      else
      if (m_nCmPerSec >= 1100)
         {
         gVoice.m_nVoiceSel = S_11;
         nDelta = m_nCmPerSec - 1100;
         }
      else
      if (m_nCmPerSec >= 1000)
         {
         gVoice.m_nVoiceSel = S_10;
         nDelta = m_nCmPerSec - 1000;
         }
      else
      if (m_nCmPerSec >= 900)
         {
         gVoice.m_nVoiceSel = S_9;
         nDelta = m_nCmPerSec - 900;
         }
      else
      if (m_nCmPerSec >= 800)
         {
         gVoice.m_nVoiceSel = S_8;
         nDelta = m_nCmPerSec - 800;
         }
      else
      if (m_nCmPerSec >= 700)
         {
         gVoice.m_nVoiceSel = S_7;
         nDelta = m_nCmPerSec - 700;
         }
      else
      if (m_nCmPerSec >= 600)
         {
         gVoice.m_nVoiceSel = S_6;
         nDelta = m_nCmPerSec - 600;
         }
      else
      if (m_nCmPerSec >= 500)
         {
         gVoice.m_nVoiceSel = S_5;
         nDelta = m_nCmPerSec - 500;
         }
      else
      if (m_nCmPerSec >= 400)
         {
         gVoice.m_nVoiceSel = S_4;
         nDelta = m_nCmPerSec - 400;
         }
      else
      if (m_nCmPerSec >= 300)
         {
         gVoice.m_nVoiceSel = S_3;
         nDelta = m_nCmPerSec - 300;
         }
      else
      if (m_nCmPerSec >= 200)
         {
         gVoice.m_nVoiceSel = S_2;
         nDelta = m_nCmPerSec - 200;
         }
      else
      if (m_nCmPerSec >= 100)
         {
         gVoice.m_nVoiceSel = S_1;
         nDelta = m_nCmPerSec - 100;
         }
      else
         {
         gVoice.m_nVoiceSel = S_0;
         nDelta = m_nCmPerSec;
         }
      gVoice.SetBitRate(gVoice.m_nVoiceSel,13000 + (nDelta<<6));
      }
      
   if (gVoice.m_nVoiceSel >= 0)
      gVoice.Trigger();
   }


void CVario::Chirp(int nCmPerSec)
   {
   int nFreqHz,bChange;
   u16 nFreqReg;

   CLAMP(nCmPerSec,-VARIO_MAX_CLIMB_RATE_DPY_CM,VARIO_MAX_CLIMB_RATE_DPY_CM);

   if (nCmPerSec < -m_nSinkThresholdCm)
      {
      m_nCmPerSec = nCmPerSec;
      nFreqHz = m_nCmPerSec < -VARIO_SINKTONE_HI_CM ? VARIO_SINK_FREQ_HI : VARIO_SINK_FREQ_LO;
      nFreqReg = VARIO_FREQ2REG(nFreqHz);
      // program regs only if coming from chirp / off mode or if freq changes
      if (m_nLoopMode == VARIO_LOOP_TIMED || nFreqReg != m_nFreqReg)
         {
         m_nLoopMode = VARIO_LOOP_CONT;
         m_nFreqReg = nFreqReg;
         m_nTotalTicks = 0;
         m_nTickCnt = 1;
         // disable chirp
         m_nSweepTime = 0;
         m_nNumSweeps = 0;
         m_nSweepDir = 1;
         // set sound length to 0 for continuous mode
         m_nSoundLen = 0;

         GBA_REG_SG11 |= 0x8000; // reset
         M_SND1CNT1_SETUP(m_nSweepTime,m_nNumSweeps,m_nSweepDir);
         M_SND1CNT2_SETUP(m_nInitVolume,m_nFadeMode,m_nVolumeSteps,m_nSoundLen,m_nDutyCycle);
         M_SND1CNT3_SETUP(m_nFreqReg, m_nLoopMode);
         }
      }
   else
   // no sound
   if (nCmPerSec <  VARIO_CORE_CLIMB_RATE_CM)
      {
      m_nCmPerSec = nCmPerSec;
      m_nTotalTicks = 0;
      m_nTickCnt = 1;
      if (m_nLoopMode == VARIO_LOOP_CONT) // program once to get out of continuous tone
         {
         GBA_REG_SG11 |= 0x8000; // reset
         m_nLoopMode = VARIO_LOOP_TIMED;
         M_SND1CNT1_SETUP(m_nSweepTime,m_nNumSweeps,m_nSweepDir);
         M_SND1CNT2_SETUP(m_nInitVolume,m_nFadeMode,m_nVolumeSteps,m_nSoundLen,m_nDutyCycle);
         M_SND1CNT3_SETUP(m_nFreqReg, m_nLoopMode);
         }
      }
   else // climbing, need to start chirping
      {
   // Assume a low steady climb rate < 1 m/s, the soundlen is programmed as 0 which is 
   // (64-0)/256 = 250mS.
   // For this low climb rate, the chirp repeat interval
   // m_nTotalTicks is 4 vario ticks. If the vario tick is 100 mS, 4 ticks is a chirp
   // repeat interval of 400mS.
   // So, chirp 250mS, follow with 150mS of silence, then repeat.
   // If the climbrate has changed by more than m_nResolutionCm during the vario tick
   // interval, we want a new chirp (with new frequency etc.) to begin on the next
   // vario tick, not after 4 ticks. 
   // If the climbrate has changed less than m_nResolutionCm, after 4 ticks, 
   // a new chirp is programmed.
   
      bChange = ((ABS(nCmPerSec - m_nCmPerSec) > m_nResolutionCm) || (m_nTickCnt == 0) || (m_nTotalTicks == 0)) ? 1 : 0;
      if (bChange)
         {
         m_nTickCnt = 0;
         m_nCmPerSec = nCmPerSec;
         m_nSweepDir  = 0;
         m_nNumSweeps = 7;
         m_nLoopMode = VARIO_LOOP_TIMED;
         // allow audible resolution of climb rate to number of chirps per second
         if (m_nCmPerSec > VARIO_CROSSOVER_CLIMB_RATE_CM)
            {
            m_nSweepTime = 3;
            m_nTotalTicks = 1;
            m_nSoundLen = 48;
            nFreqHz = VARIO_CROSSOVER_CLIMB_FREQ + ((m_nCmPerSec-VARIO_CROSSOVER_CLIMB_RATE_CM)*(VARIO_MAX_CLIMB_FREQ-VARIO_CROSSOVER_CLIMB_FREQ))/(VARIO_MAX_CLIMB_RATE_DPY_CM-VARIO_CROSSOVER_CLIMB_RATE_CM);
            m_nFreqReg = VARIO_FREQ2REG(nFreqHz);
            }
         else
            {
            nFreqHz = VARIO_MIN_CLIMB_FREQ + ((m_nCmPerSec-VARIO_CORE_CLIMB_RATE_CM)*(VARIO_CROSSOVER_CLIMB_FREQ-VARIO_MIN_CLIMB_FREQ))/(VARIO_CROSSOVER_CLIMB_RATE_CM-VARIO_CORE_CLIMB_RATE_CM);
            m_nFreqReg = VARIO_FREQ2REG(nFreqHz);
            if (m_nCmPerSec > 200)
               {
               m_nSweepTime = 5;
               m_nTotalTicks = 2;
               m_nSoundLen = 32;
               }
            else
            if (m_nCmPerSec > 100)
               {
               m_nSweepTime = 6;
               m_nTotalTicks = 3;
               m_nSoundLen  = 16;
               }
            else
               {
               m_nSweepTime = 7;
               m_nTotalTicks  = 4;
               m_nSoundLen  = 0;
               }
            }
         GBA_REG_SG11 |= 0x8000; // reset
         M_SND1CNT1_SETUP(m_nSweepTime,m_nNumSweeps,m_nSweepDir);
         M_SND1CNT2_SETUP(m_nInitVolume,m_nFadeMode,m_nVolumeSteps,m_nSoundLen,m_nDutyCycle);
         M_SND1CNT3_SETUP(m_nFreqReg, m_nLoopMode);
         }
      }
      
   if (m_nTotalTicks)
      {
      if (++m_nTickCnt == m_nTotalTicks)
         m_nTickCnt = 0;
      }
   }

/*
 * $Header: $
 */

