const char szCopyRight[] = "Copyright (c) Hari Nair, Bangalore, India 2006";
const char szVersion[] = "Gameboy Advance Altimeter-Vario Version 2.30";


#include <mygba.h>
#include <hel2.h>
#include "def.h"
#include "text.h"
#include "vario.h"
#include "tmr.h"
#include "i2c.h"
#include "nvd.h"
#include "rtc.h"
#include "sensor.h"
#include "utils.h"
#include "ui.h"
#include "bgtext.h"
#include "voice.h"

#ifdef USE_SPLASH_SCREEN
#include "gfx/splash.pal.c"
#include "gfx/splash.raw.c"
#endif

CSensor gSensor;
CVario  gVario;
CNVD    gNVD;
CTimer  gTimer;
CRTC    gRTC;
CI2C    gI2C;
#ifdef SENSOR_SCP1000
CI2C    gI2CB;
#endif
CVoice  gVoice;


int gnFrameCnt;
int gnMode;

FLIGHT_DATA gFD;

int main(void)
   {
   ham_Init();

#ifdef SENSOR_MS5534
   gI2C.Init(PORT_SI,PORT_SO);
#endif
#ifdef SENSOR_FOSP01
   gI2C.Init(PORT_SC,PORT_SD);
#endif
#ifdef SENSOR_SCP1000
   gI2CB.Init(PORT_SC,PORT_SD);
#endif
   
   gRTC.Init();
   gRTC.ReadData();
   gTimer.Init(gRTC.m_dow,gRTC.m_hrs,gRTC.m_min,gRTC.m_sec);
   gVario.Init();
   gVario.SetMode(VARIO_MODE_VOICE);

#ifdef USE_SPLASH_SCREEN
   // hel_Splash() works only in bitmap-modes, these are mode 3, 4 and 5
   ham_SetBgMode(4);
   hel_Splash((void*)splash_Bitmap,         // pBitmap
              (u16*)splash_Palette,        // pPalette
              RGB_SET(0,0,0),               // black
              RGB_SET(31,31,31),               // white
              50,                               // Blend delay
              1000,                               // Display delay
              50,                              // Continue delay
              COMPRESSION_TYPE_NONE,
              0);                             
#endif

   ResetBkg();


   gSensor.Init();

#ifdef SENSOR_FOSP01
   ham_DrawText(0,0,"C1 %d",gSensor.m_C1);
   ham_DrawText(0,1,"C2 %d",gSensor.m_C2);
   ham_DrawText(0,2,"C3 %d",gSensor.m_C3);
   ham_DrawText(0,3,"C4 %d",gSensor.m_C4);
   ham_DrawText(0,4,"C5 %d",gSensor.m_C5);
   ham_DrawText(0,5,"C6 %d",gSensor.m_C6);
   ham_DrawText(0,6,"C7 %d",gSensor.m_C7);
   ham_DrawText(0,7,"A %d",gSensor.m_A);
   ham_DrawText(0,8,"B %d",gSensor.m_B);
   ham_DrawText(0,9,"C %d",gSensor.m_C);
   ham_DrawText(0,10,"D %d",gSensor.m_D);

   gnFrameCnt = 0xFFFFF;
   while(gnFrameCnt--);
#endif

   if (gNVD.BadChecksum())
      {
      gSensor.m_nPZTblSel   = SENSOR_DFL_PZTBL_SEL;
      gSensor.m_nWindowSecs = SENSOR_DFL_WINDOW_SECS;
      gSensor.m_nWindowSamples = gSensor.m_nWindowSecs*SENSOR_SAMPLES_PER_SEC;
      gSensor.m_nKLPFC      = SENSOR_DFL_KLPFC;
      gVario.m_nSinkThresholdCm = VARIO_DFL_SINKTONE_THRESHOLD_CM;
      gVario.m_nResolutionCm  = VARIO_DFL_RESOLUTION;
      gVario.m_nMode = VARIO_MODE_CHIRP;
      gNVD.WriteDefaults();
      }
   else
      {
      gSensor.m_nPZTblSel = gNVD.ReadPZTblSel();
      gSensor.m_nWindowSecs = gNVD.ReadWindowSecs();
      gSensor.m_nWindowSamples = gSensor.m_nWindowSecs*SENSOR_SAMPLES_PER_SEC;
      gSensor.m_nKLPFC = gNVD.ReadKLPFC();
      gVario.m_nSinkThresholdCm = gNVD.ReadSinkThresholdCm();
      gVario.m_nResolutionCm  = gNVD.ReadVarioRes();
      gVario.m_nMode = gNVD.ReadVarioAudioMode();
      gNVD.m_nCurInx = gNVD.ReadCurInx();
      }

    gnFrameCnt = 0;
    ham_InitPad();
    ham_StartIntHandler(INT_TYPE_VBL, (void *)&VblFunc);

    InitLogMode();

    while(1)
        {
        switch (gnMode)
           {
           case MODE_LOG :
           LogEventHandler();
           break;

           case MODE_SETTINGS :
           SetEventHandler();
           break;

           case MODE_SIMULATION :
           SimEventHandler();
           break;

           case MODE_RUNNING :
           default :
           RunEventHandler();
           break;
           }
        }
    return 0;
    }



/*
 * $Header: $
 * 2.00 Optimized software to accommodate both MS5534B and SCP1000 sensors
 *      Simplified sensor sampling for MS5534B
 *      New sensor code for SCP1000
 *      New user-interface, color-coded background for climb and sink
 *      General cleanup
 * 2.01 SCP1000 code now uses one sensor for altitude/temp readings, both
 *      sensors for vario calculation.
 * 2.02 Fixed bug in gSensor.m_nWindowSamples initialization
 * 2.10 Fixed bug in RTC sec/dom update display
 *      Vario neutral zone display colour changed from gray to yellow
 *      Smooth scrolling for climb
 *      New anti-aliased fonts seem to work ok with the red/yellow/blue
 *      changing background
 * 2.11 Now vario scroll tape divisions are no longer compressed, range
 *      is now -16m/s to +16m/s. Deleted the screen in front of the vario
 *      tape. Background does not change colour any more...
 * 2.12 Option of selecting two Pressure->Altitude tables when compiling,
 *      for reference temperature of 15Celsius or 25Celsius. See sensor.cpp
 * 2.13 Automatic reference temperature "compensation", no need for selecting
 *      Pressure->Altitude tables during compile any more.
 * 2.20 Added support for the FOSP01 pressure sensor - unfortunately no
 *      working sample yet...
 * 2.21 Display altitude MSL range now -999M to 999M. Number of pressure
 *      altitude tables increased to 100. 
 * 2.22 Removed automatic temperature compensation as this doesn't seem to 
 *      consistently be an improvement.
 *      Simulation mode now shows altitude, altitude above launch, flight
 *      duration, and temperature consistent with the climb and sink rates
 *      over time - thanks to Yuriy Kozhynov for this feature.
 * 2.30 Introducing voiced vario option. Timer 0 and 1 utilized for voice
 *      DMA, Timer 2 now used for the timing functions. Simulation mode
 *      screen background now yellow. Real-time Display screen flight duration
 *      timer initial display now consistent with the simulation mode timer display.
 */

